import logging
from copy import deepcopy
from enum import Enum
from json import JSONEncoder, dumps

log = logging.getLogger(__name__)


class Issue(object):
    def __init__(self, category, name, severity, description, line_number=None, file_object=None, apk_exploit_dict=None):
        """
        Create a vulnerability, used by Plugins.

        :param str category: category to put the vulnerability in the report.
        :param Severity severity: severity of the vulnerability, Severity.INFO, Severity.VULNERABILITY, Severity.ERROR, or Severity.WARNING.
        :param Tuple[int, int] line_number: line number of where the vulnerability was found.
        :param str file_object: file where the vulnerability occurred.
        :param Dict apk_exploit_dict: dictionary containing information that is needed to build the exploit apk
        """
        self.category = category

        # convert severity to its enum
        if not isinstance(severity, Severity):
            if isinstance(severity, str):
                if severity.lower() == "info":
                    severity = Severity.INFO
                elif severity.lower() == "vulnerability":
                    severity = Severity.VULNERABILITY
                elif severity.lower() == "error":
                    severity = Severity.ERROR
                elif severity.lower() == "warning":
                    severity = Severity.WARNING
                else:
                    log.info("Severity is not set for issue. Setting severity to a warning.")
                    severity = Severity.WARNING
            else:
                log.info("Severity is not set for issue. Setting severity to a warning.")
                severity = Severity.WARNING

        self.severity = severity
        self.description = description
        self.name = name
        self.line_number = line_number
        self.file_object = file_object
        self.apk_exploit_dict = apk_exploit_dict

    def __repr__(self):
        return ("Issue(category={}, name={}, severity={}, description={}, line_number={}, "
                "file_object={}, apk_exploit_dict={})".format(self.category, self.name, self.severity,
                                                              self.description, self.line_number, self.file_object,
                                                              self.apk_exploit_dict))

    def __hash__(self):
        return hash((self.name, self.file_object, self.line_number))


class Severity(Enum):
    INFO = 0
    WARNING = 1
    ERROR = 2
    VULNERABILITY = 3


class IssueEncoder(JSONEncoder):
    def default(self, issue):
        if isinstance(issue, Issue):
            working_dict = deepcopy(issue.__dict__)
            working_dict['severity'] = working_dict['severity'].name
            return working_dict
        else:
            log.error('Error converting issue %r to JSON %s', issue, type(issue))
            return


def issue_json(value):
    try:
        return dumps(value, cls=IssueEncoder)
    except TypeError:
        log.exception('Error encoding to JSON')
        return dumps('Error encoding to JSON')



from qark.scanner.plugin import ManifestPlugin
from qark.issue import Severity, Issue

import logging

log = logging.getLogger(__name__)


class DebuggableManifest(ManifestPlugin):
    def __init__(self):
        super(DebuggableManifest, self).__init__(category="manifest", name="Manifest is manually set to debug",
                                                 description=(
                                                     "The android:debuggable flag is manually set to true in the"
                                                     " AndroidManifest.xml. This will cause your application to be debuggable "
                                                     "in production builds and can result in data leakage "
                                                     "and other security issues. It is not necessary to set the "
                                                     "android:debuggable flag in the manifest, it will be set appropriately "
                                                     "automatically by the tools. More info: "
                                                     "http://developer.android.com/guide/topics/manifest/application-element.html#debug"))

        self.severity = Severity.VULNERABILITY

    def run(self):
        application_sections = self.manifest_xml.getElementsByTagName("application")
        for application in application_sections:
            try:
                if application.attributes["android:debuggable"].value.lower() == "true":
                    self.issues.append(Issue(category=self.category, severity=self.severity,
                                             name=self.name, description=self.description,
                                             file_object=self.manifest_path))
            except (KeyError, AttributeError):
                log.debug("Application section does not have debuggable flag, continuing")
                continue


plugin = DebuggableManifest()



import logging

from javalang.tree import MethodInvocation

from qark.issue import Severity, Issue
from qark.scanner.plugin import JavaASTPlugin

log = logging.getLogger(__name__)

LOAD_DATA_WITH_BASE_URL_DESCRIPTION = (
    "This webView sets the BaseURL. You should verify that this is only loading content "
    "from this domain. Loading content from a domain you do not control, or using "
    "plain-text HTTP, leaves this vulnerable to injection attacks against the BaseURL "
    "domain."
)


class LoadDataWithBaseURL(JavaASTPlugin):
    """This plugin checks if the `loadDataWithBaseURL` method is called."""
    def __init__(self):
        super(LoadDataWithBaseURL, self).__init__(category="webview", name="BaseURL set for Webview",
                                                  description=LOAD_DATA_WITH_BASE_URL_DESCRIPTION)

    def run(self):
        for _, method_invocation in self.java_ast.filter(MethodInvocation):
            if method_invocation.member == "loadDataWithBaseURL" and len(method_invocation.arguments) == 5:
                self.issues.append(Issue(category=self.category, name=self.name, severity=Severity.WARNING,
                                         description=self.description, line_number=method_invocation.position,
                                         file_object=self.file_path))


plugin = LoadDataWithBaseURL()



import logging

from qark.issue import Severity
from qark.plugins.webview.helpers import webview_default_vulnerable
from qark.scanner.plugin import JavaASTPlugin

log = logging.getLogger(__name__)

SET_ALLOW_FILE_ACCESS_DESCRIPTION = (
    "File system access is enabled in this WebView. If untrusted data is used to specify the URL opened by this "
    "WebView, a malicious app or site may be able to read your app's private files, if it returns the response to "
    "them. To validate this vulnerability, load the following local file in this "
    "WebView: qark/poc/html/FILE_SYS_WARN.html"
)


class SetAllowFileAccess(JavaASTPlugin):
    """This plugin checks if the webview calls setAllowFileAccess(false), otherwise the webview is vulnerable
    (defaults to true)."""
    def __init__(self):
        super(SetAllowFileAccess, self).__init__(category="webview", name="Webview enables file access",
                                                 description=SET_ALLOW_FILE_ACCESS_DESCRIPTION)
        self.severity = Severity.WARNING

    def run(self):
        self.issues.extend(webview_default_vulnerable(self.java_ast, method_name="setAllowFileAccess", issue_name=self.name,
                                                      description=self.description, file_object=self.file_path,
                                                      severity=self.severity))


plugin = SetAllowFileAccess()



#!/usr/bin/env python

from __future__ import absolute_import

import logging
import logging.config

import click
import os

from qark.apk_builder import APKBuilder
from qark.decompiler.decompiler import Decompiler
from qark.report import Report
from qark.scanner.scanner import Scanner
from qark.utils import environ_path_variable_exists

DEBUG_LOG_PATH = os.path.join(os.getcwd(),
                              "qark_debug.log")

# Environment variable names for the SDK
ANDROID_SDK_HOME = "ANDROID_SDK_HOME"
ANDROID_HOME = "ANDROID_HOME"
ANDROID_SDK_ROOT = "ANDROID_SDK_ROOT"

logger = logging.getLogger(__name__)


@click.command()
@click.option("--sdk-path", type=click.Path(exists=True, file_okay=False, resolve_path=True),
              help="Path to the downloaded SDK directory if already downloaded. "
                   "Only necessary if --exploit-apk is passed. If --exploit-apk is passed and this flag is not passed,"
                   "QARK will attempt to use the ANDROID_SDK_HOME, ANDROID_HOME, ANDROID_SDK_ROOT "
                   "environment variables (in that order) for a path.")
@click.option("--build-path", type=click.Path(resolve_path=True, file_okay=False),
              help="Path to place decompiled files and exploit APK.", default="build", show_default=True)
@click.option("--debug/--no-debug", default=False, help="Show debugging statements (helpful for issues).",
              show_default=True)
@click.option("--apk", "source", help="APK to decompile and run static analysis. If passed, "
                                      "the --java option is not used.",
              type=click.Path(exists=True, resolve_path=True, file_okay=True, dir_okay=True))
@click.option("--java", "source", type=click.Path(exists=True, resolve_path=True, file_okay=True, dir_okay=True),
              help="A directory containing Java code, or a Java file, to run static analysis. If passed,"
                   "the --apk option is not used.")
@click.option("--report-type", type=click.Choice(["html", "xml", "json", "csv"]),
              help="Type of report to generate along with terminal output.", default="html", show_default=True)
@click.option("--exploit-apk/--no-exploit-apk", default=False,
              help="Create an exploit APK targetting a few vulnerabilities.", show_default=True)
@click.version_option()
@click.pass_context
def cli(ctx, sdk_path, build_path, debug, source, report_type, exploit_apk):
    if not source:
        click.secho("Please pass a source for scanning through either --java or --apk")
        click.secho(ctx.get_help())
        return

    if exploit_apk:

        if not sdk_path:
            # Try to set the SDK from environment variables if they exist
            # Follows the guidelines from https://developer.android.com/studio/command-line/variables
            if environ_path_variable_exists(ANDROID_SDK_HOME):
                sdk_path = os.environ[ANDROID_SDK_HOME]

            elif environ_path_variable_exists(ANDROID_HOME):
                sdk_path = os.environ[ANDROID_HOME]

            elif environ_path_variable_exists(ANDROID_SDK_ROOT):
                sdk_path = os.environ[ANDROID_SDK_ROOT]

            else:
                click.secho("Please provide path to android SDK if building exploit APK.")
                return

    # Debug controls the output to stderr, debug logs are ALWAYS stored in `qark_debug.log`
    if debug:
        level = "DEBUG"
    else:
        level = "INFO"

    initialize_logging(level)

    click.secho("Decompiling...")
    decompiler = Decompiler(path_to_source=source, build_directory=build_path)
    decompiler.run()

    click.secho("Running scans...")
    if decompiler.source_code:
        scanner = Scanner(manifest_path=decompiler.manifest_path, path_to_source=decompiler.path_to_source)
    else:
        scanner = Scanner(manifest_path=decompiler.manifest_path, path_to_source=decompiler.build_directory)
    scanner.run()
    click.secho("Finish scans...")

    click.secho("Writing report...")
    report = Report(issues=set(scanner.issues))
    report_path = report.generate(file_type=report_type)
    click.secho("Finish writing report to {report_path} ...".format(report_path=report_path))

    if exploit_apk:
        click.secho("Building exploit APK...")
        exploit_builder = APKBuilder(exploit_apk_path=build_path, issues=scanner.issues, apk_name=decompiler.apk_name,
                                     manifest_path=decompiler.manifest_path, sdk_path=sdk_path)
        exploit_builder.build()
        click.secho("Finish building exploit APK...")


def initialize_logging(level):
    """Creates two root handlers, one to file called `qark_debug.log` and one to stderr"""
    handlers = {
        "stderr_handler": {
            "level": level,
            "class": "logging.StreamHandler"
        }
    }
    loggers = ["stderr_handler"]

    if level == "DEBUG":
        handlers["debug_handler"] = {
            "level": "DEBUG",
            "class": "logging.FileHandler",
            "filename": DEBUG_LOG_PATH,
            "mode": "w",
            "formatter": "standard"
        }
        loggers.append("debug_handler")

    logging.config.dictConfig({
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": {
            "standard": {
                'format': '%(asctime)s [%(levelname)s] %(name)s: %(message)s'
            }
        },
        "handlers": handlers,
        "loggers": {
            "": {
                "handlers": handlers,
                "level": "DEBUG",
                "propagate": True
            }
        }
    })

    if level == "DEBUG":
        logger.debug("Debug logging enabled")



from qark.scanner.plugin import ManifestPlugin

import logging

from qark.issue import Severity, Issue
from qark.plugins.manifest_helpers import get_min_sdk

log = logging.getLogger(__name__)

SIGNATURE_OR_SIGNATURE_OR_SYSTEM_DESCRIPTION = (
    "This permission can be obtained by malicious apps installed prior to this "
    "one, without the proper signature. Applicable to Android Devices prior to "
    "L (Lollipop). More info: "
    "https://github.com/commonsguy/cwac-security/blob/master/PERMS.md"
)
SIGNATURE_OR_SIGNATURE_OR_SYSTEM_SEVERITY = Severity.WARNING

DANGEROUS_PERMISSION_DESCRIPTION = (
    "This permission can give a requesting application access to private user data or control over the "
    "device that can negatively impact the user."
)


class CustomPermissions(ManifestPlugin):
    def __init__(self):
        super(CustomPermissions, self).__init__(category="manifest",
                                                name="Custom permissions are enabled in the manifest",
                                                description=(
                                                    "This permission can be obtained by malicious apps installed prior to this "
                                                    "one, without the proper signature. Applicable to Android Devices prior to "
                                                    "L (Lollipop). More info: "
                                                    "https://github.com/commonsguy/cwac-security/blob/master/PERMS.md"))

        self.severity = Severity.WARNING

    def run(self):
        permission_sections = self.manifest_xml.getElementsByTagName("permission")
        for permission in permission_sections:
            try:
                protection_level = permission.attributes["android:protectionLevel"].value
            except KeyError:
                continue

            if protection_level in ("signature", "signatureOrSystem"):
                if self.min_sdk < 21:
                    self.issues.append(Issue(category=self.category,
                                             severity=SIGNATURE_OR_SIGNATURE_OR_SYSTEM_SEVERITY,
                                             name=self.name,
                                             description=SIGNATURE_OR_SIGNATURE_OR_SYSTEM_DESCRIPTION,
                                             file_object=self.manifest_path))

            elif protection_level == "dangerous":
                self.issues.append(Issue(category=self.category,
                                         severity=Severity.INFO,
                                         name=self.name,
                                         description=DANGEROUS_PERMISSION_DESCRIPTION,
                                         file_object=self.manifest_path))


plugin = CustomPermissions()



import logging

import javalang
from javalang.tree import MethodInvocation

from qark.issue import Severity, Issue
from qark.plugins.webview.helpers import valid_set_method_bool
from qark.scanner.plugin import JavaASTPlugin

log = logging.getLogger(__name__)

JAVASCRIPT_REMOTE_DEBUGGING = (
    "Enabling webview remote debugging is insecure."
)

class RemoteDebugging(JavaASTPlugin):
    """This plugin checks if the `setWebContentsDebuggingEnabled` method is called with a value of `true`"""
    def __init__(self):
        super(RemoteDebugging, self).__init__(category="webview", name="Remote debugging enabled in Webview",
                                              description=JAVASCRIPT_REMOTE_DEBUGGING)
    def run(self):
        for _, method_invocation in self.java_ast.filter(MethodInvocation):
            if valid_set_method_bool(method_invocation, str_bool="true", method_name="setWebContentsDebuggingEnabled"):
                self.issues.append(Issue(category=self.category, name=self.name, severity=Severity.WARNING,
                                         description=self.description, line_number=method_invocation.position,
                                         file_object=self.file_path))

plugin = RemoteDebugging()



from xml.etree import ElementTree
from xml.dom import minidom
import logging

from qark.xml_helpers import get_manifest_out_of_files

log = logging.getLogger(__name__)


def get_package_from_manifest(manifest_path):
    """
    Gets the package name from an AndroidManifest.xml file path

    :param manifest_path: path to android manifest
    :return: package name if exists
    """
    try:
        manifest_xml = ElementTree.parse(manifest_path)
    except IOError:
        raise

    return manifest_xml.getroot().attrib.get("package")


def get_min_sdk(manifest_xml, files=None):
    """
    Given the manifest as a `minidom.parse`'d object or path to manifest,
    try to get the minimum SDK the manifest specifies.

    :param manifest_xml: object after parsing the XML
    :param Set[str] files: list of files received from Scanner
    :return: int of the version if it exists, else 1 (the default)
    """
    if manifest_xml is None and files:
        manifest_xml = get_manifest_out_of_files(files)

    if isinstance(manifest_xml, str):
        manifest_xml = minidom.parse(manifest_xml)

    # TODO: try to get SDK from gradle file
    try:
        sdk_section = manifest_xml.getElementsByTagName("uses-sdk")[0]
    except IndexError:
        log.debug("Unable to get uses-sdk section")
        return 1

    try:
        return int(sdk_section.attributes["android:minSdkVersion"].value)
    except (KeyError, AttributeError):
        log.debug("Unable to get minSdkVersion from manifest")
        return 1


def get_target_sdk(manifest_xml, files=None):
    """
    Given the manifest as a `minidom.parse`'d object, try to get the target SDK the manifest specifies.

    :param manifest_xml: object after parsing the XML
    :param Set[str] files: list of files received from Scanner
    :return: int of the version if it exists, else 1 (the default)
    """
    if manifest_xml is None and files:
        manifest_xml = get_manifest_out_of_files(files)

    if isinstance(manifest_xml, str):
        manifest_xml = minidom.parse(manifest_xml)

    # TODO: try to get SDK from gradle file
    try:
        sdk_section = manifest_xml.getElementsByTagName("uses-sdk")[0]
    except IndexError:
        log.debug("Unable to get uses-sdk section")
        return 1

    try:
        return int(sdk_section.attributes["android:targetSdkVersion"].value)
    except (KeyError, AttributeError):
        log.debug("Unable to get targetSdkVersion from manifest")
        return 1




import logging

import javalang
from javalang.tree import ClassDeclaration, MethodDeclaration

from qark.issue import Severity, Issue
from qark.scanner.plugin import JavaASTPlugin

log = logging.getLogger(__name__)

INSECURE_FUNCTIONS_DESCRIPTION = (
    "The Content provider API provides a method call. The framework does no permission checking on this "
    "entry into the content provider besides the basic ability for the application to get access to the provider"
    " at all. Any implementation of this method must do its own permission checks on incoming calls to make sure "
    "they are allowed. Failure to do so will allow unauthorized components to interact with the content provider. "
    "Reference: https://bitbucket.org/secure-it-i/android-app-vulnerability-benchmarks/src/d5305b9481df3502e60e98fa352d5f58e4a69044/ICC/WeakChecksOnDynamicInvocation-InformationExposure/?at=master"
)

INSECURE_FUNCTIONS_NAMES = ("call",)


class InsecureFunctions(JavaASTPlugin):
    def __init__(self):
        super(InsecureFunctions, self).__init__(category="file", name="Insecure functions found",
                                                description=INSECURE_FUNCTIONS_DESCRIPTION)
        self.severity = Severity.WARNING

    def run(self):
        for _, class_declaration in self.java_ast.filter(ClassDeclaration):
            for _, method_declaration_in_class in class_declaration.filter(MethodDeclaration):

                if method_declaration_in_class.name in INSECURE_FUNCTIONS_NAMES:
                    self.issues.append(Issue(
                        category=self.category, severity=self.severity, name=self.name,
                        description=self.description,
                        file_object=self.file_path,
                        line_number=method_declaration_in_class.position)
                    )


plugin = InsecureFunctions()



from qark.scanner.plugin import JavaASTPlugin, ManifestPlugin
from qark.issue import Severity, Issue

import logging
import re

import javalang
from javalang.tree import MethodInvocation


log = logging.getLogger(__name__)

BROADCAST_METHODS = (
    "sendBroadcast",
    "sendBroadcastAsUser",
    "sendOrderedBroadcast",
    "sendOrderedBroadcastAsUser",
    "sendStickyBroadcast",
    "sendStickyBroadcastAsUser",
    "sendStickyOrderedBroadcast",
    "sendStickyOrderedBroadcastAsUser",
)
STICKY_BROADCAST_METHODS = (
    "sendStickyBroadcast",
    "sendStickyBroadcastAsUser",
    "sendStickyOrderedBroadcast",
    "sendStickyOrderedBroadcastAsUser",
)
LOCAL_BROADCAST_IMPORTS = (
    "android.support.v4.content.LocalBroadcastManager",
    "android.support.v4.content.*",
    "android.support.v4.*",
    "android.support.*",
    "android.*",
)
BROADCAST_WITHOUT_RECEIVER = (
    "A broadcast, {broadcast_type} which does not specify the receiverPermission. "
    "This means any application on the device can receive this broadcast. You should investigate this "
    "for potential data leakage."
)
BROADCAST_WITH_RECEIVER = (
    "A broadcast, {broadcast_type} which specifies the receiverPermission, "
    "but depending on the protection level of the permission (on the receiving app side), may "
    "still be vulnerable to interception, if the protection level of the permission is not set "
    "to signature or signatureOrSystem. You should investigate this for potential data leakage."
)
BROADCAST_WITH_RECEIVER_UNDER_21 = (
    "A broadcast, {broadcast_type} which specifies the "
    "receiverPermission, but may still be vulnerable to interception, due to the permission squatting vulnerability "
    "in API levels before 21. This means any application, installed prior to the expected receiver(s) on the device "
    "can potentially receive this broadcast. You should investigate this for potential data leakage."
)
STICKY_BROADCAST = (
    "A sticky broadcast, {broadcast_type}. These should not be used, as they "
    "rovide no security (anyone can access them), no protection (anyone can modify them), and many other problems. "
    "For more info: http://developer.android.com/reference/android/content/Context.html"
)


class SendBroadcastReceiverPermission(JavaASTPlugin, ManifestPlugin):
    """
    This plugin checks certain broadcast methods to see if they are using an insecure version,
    based on number of arguments.
    """
    def __init__(self):
        super(SendBroadcastReceiverPermission, self).__init__(category="broadcast",
                                                              name="Send Broadcast Receiver Permission")
        self.severity = Severity.WARNING
        self.current_file = None
        self.manifest_xml = None
        self.below_min_sdk_21 = False

    def run(self):
        self.below_min_sdk_21 = self.min_sdk < 21
        self.current_file = self.file_path
        if not re.search("({broadcasts})".format(broadcasts="|".join(BROADCAST_METHODS)), self.file_contents):
            return

        for _, method_invocation in self.java_ast.filter(MethodInvocation):
            self._check_method_invocation(method_invocation, self.java_ast.imports)

    def _check_method_invocation(self, method_invocation, imports):
        """
        Checks the type of method that is being called and the amount of arguments and adds it to the issues list if it
        results in a warning or vulnerability.

        :param javalang.tree.MethodInvocation method_invocation: method being called
        :param list imports: list of imports
        """
        method_name = method_invocation.member
        num_arguments = len(method_invocation.arguments)
        if method_name in BROADCAST_METHODS:
            name = None
            description = None

            if method_name == "sendBroadcast":
                if num_arguments == 1 and not has_local_broadcast_imported(import_tree=imports):
                    name = "Broadcast sent without receiverPermission"
                    description = BROADCAST_WITHOUT_RECEIVER
                elif num_arguments == 2:
                    if self.below_min_sdk_21:
                        name = "Broadcast sent with receiverPermission with minimum SDK under 21"
                        description = BROADCAST_WITH_RECEIVER_UNDER_21
                    else:
                        name = "Broadcast sent with receiverPermission"
                        description = BROADCAST_WITH_RECEIVER

            elif method_name == "sendBroadcastAsUser":
                if num_arguments == 2:
                    name = "Broadcast sent as specific user without receiverPermission"
                    description = BROADCAST_WITH_RECEIVER_UNDER_21
                elif num_arguments == 3:
                    if self.below_min_sdk_21:
                        name = "Broadcast sent as specific user with receiverPermission with minimum SDK under 21"
                        description = BROADCAST_WITH_RECEIVER_UNDER_21
                    else:
                        name = "Broadcast sent as specific user with receiverPermission"
                        description = BROADCAST_WITH_RECEIVER

            elif method_name == "sendOrderedBroadcast":
                if num_arguments in (2, 7):
                    if self.below_min_sdk_21:
                        name = "Ordered broadcast sent with receiverPermission with minimum SDK under 21"
                        description = BROADCAST_WITH_RECEIVER_UNDER_21
                    else:
                        name = "Ordered broadcast sent with receiverPermission"
                        description = BROADCAST_WITH_RECEIVER

            elif method_name == "sendOrderedBroadcastAsUser":
                if num_arguments == 7:
                    if self.below_min_sdk_21:
                        name = "Ordered broadcast sent with receiverPermission with minimum SDK under 21"
                        description = BROADCAST_WITH_RECEIVER_UNDER_21
                    else:
                        name = "Ordered broadcast sent with receiverPermission"
                        description = BROADCAST_WITH_RECEIVER

            elif method_name in STICKY_BROADCAST_METHODS:
                self._add_issue(name="Sticky broadcast sent", description=STICKY_BROADCAST,
                                broadcast_type=method_name, severity=Severity.VULNERABILITY,
                                line_number=method_invocation.position)

            if name is not None and description is not None:
                # create vulnerabilities for everything that is not a sticky broadcast
                self._add_issue(name=name, description=description,
                                broadcast_type=method_name, line_number=method_invocation.position)

    def _add_issue(self, name, description, broadcast_type, severity=Severity.WARNING, line_number=None):
        """
        Helper method to append issues to the plugin.

        :param str name: issue name
        :param str description: issue description
        :param str broadcast_type: type of broadcast from `BROADCAST_METHODS`
        :param Severity severity: issue severity
        """
        self.issues.append(Issue(
            category=self.category, severity=severity, name=name,
            description=description.format(broadcast_type=broadcast_type),
            file_object=self.current_file,
            line_number=line_number)
        )


def has_local_broadcast_imported(import_tree):
    """
    Helper function to determine if the import tree contains any broadcast import.

    :param list import_tree: `javalang` import tree
    :return: True if import tree has broadcast import, else False
    :rtype: bool
    """
    return any(import_declaration.path in LOCAL_BROADCAST_IMPORTS for import_declaration in import_tree)


plugin = SendBroadcastReceiverPermission()



"""This plugin detects if a method ``registerReceiver`` is being called with a min_sdk of less than 14.

ICE_CREAM_SANDWICH properly registers receivers so anything >= API level 14 is not vulnerable."""

import logging

from javalang.tree import MethodInvocation
from qark.issue import Issue, Severity
from qark.scanner.plugin import JavaASTPlugin, ManifestPlugin

log = logging.getLogger(__name__)

DYNAMIC_BROADCAST_RECEIVER_DESCRIPTION = (
    "Application that register a broadcast receiver dynamically is vulnerable to granting unrestricted access to the "
    "broadcast receiver. The receiver will be called with any broadcast Intent that matches filter."
    " Reference: https://developer.android.com/reference/android/"
    "content/Context.html#registerReceiver(android.content.BroadcastReceiver, android.content.IntentFilter)"
)

JAVA_DYNAMIC_BROADCAST_RECEIVER_METHOD = 'registerReceiver'


class DynamicBroadcastReceiver(JavaASTPlugin, ManifestPlugin):

    def __init__(self):
        super(DynamicBroadcastReceiver, self).__init__(category="broadcast", name="Dynamic broadcast receiver found",
                                                       description=DYNAMIC_BROADCAST_RECEIVER_DESCRIPTION)
        self.severity = Severity.VULNERABILITY

    def _process(self, tree, java_file):
        for _, method_invocation in tree.filter(MethodInvocation):
                if method_invocation.member == JAVA_DYNAMIC_BROADCAST_RECEIVER_METHOD and self.min_sdk < 14:
                    self.issues.append(Issue(
                        category=self.category, severity=self.severity, name=self.name,
                        description=DYNAMIC_BROADCAST_RECEIVER_DESCRIPTION,
                        file_object=java_file,
                        line_number=method_invocation.position)
                    )

    def run(self):
        self._process(tree=self.java_ast, java_file=self.file_path)


plugin = DynamicBroadcastReceiver()



import logging

import javalang
from javalang.tree import MethodInvocation

from qark.issue import Severity, Issue
from qark.plugins.helpers import java_files_from_files
from qark.plugins.webview.helpers import valid_set_method_bool
from qark.scanner.plugin import JavaASTPlugin

log = logging.getLogger(__name__)

JAVASCRIPT_ENABLED_DESCRIPTION = (
    "While not a vulnerability by itself, it appears this app has JavaScript enabled in "
    "the WebView: If this is not expressly necessary, you should disable "
    "it, to prevent the possibility of XSS (cross-site scripting) attacks. More info: "
    "http://developer.android.com/guide/practices/security.html To validate this "
    "vulnerability, load the following local file in this "
    "WebView: qark/poc/html/JS_WARNING.html"
)


class JavascriptEnabled(JavaASTPlugin):
    """This plugin checks if the `setJavaScriptEnabled` method is called with a value of `true`"""
    def __init__(self):
        super(JavascriptEnabled, self).__init__(category="webview", name="Javascript enabled in Webview",
                                                description=JAVASCRIPT_ENABLED_DESCRIPTION)

    def run(self):
        for _, method_invocation in self.java_ast.filter(MethodInvocation):
            if valid_set_method_bool(method_invocation, str_bool="true", method_name="setJavaScriptEnabled"):
                self.issues.append(Issue(category=self.category, name=self.name, severity=Severity.WARNING,
                                         description=self.description, line_number=method_invocation.position,
                                         file_object=self.file_path))


plugin = JavascriptEnabled()



import logging
import os
import platform
import re
import shlex
import shutil
import stat
import subprocess
import zipfile
from multiprocessing.pool import ThreadPool

from qark.decompiler.external_decompiler import DECOMPILERS, LIB_PATH
from qark.utils import is_java_file

log = logging.getLogger(__name__)

OS = platform.system()
JAVA_VERSION_REGEX = '"(\d+\.\d+\..+)\"'
APK_TOOL_PATH = os.path.join(LIB_PATH, "apktool")

DEX2JAR_NAME = "dex2jar-2.0"
DEX2JAR_PATH = os.path.join(LIB_PATH, DEX2JAR_NAME)
DEX2JAR_EXTENSION = "sh" if OS != "Windows" else "bat"
DEX2JAR_EXECUTABLE = "d2j-dex2jar.{extension}".format(extension=DEX2JAR_EXTENSION)
DEX2JAR_INVOKE = "d2j_invoke.{extension}".format(extension=DEX2JAR_EXTENSION)

DECOMPILERS_PATH = os.path.join(LIB_PATH, "decompilers")

APK_TOOL_COMMAND = ("java -Djava.awt.headless=true -jar {apktool_path}/apktool.jar "
                    "d {path_to_source} --no-src --force -m --output {build_directory}")
DEX2JAR_COMMAND = "{dex2jar_path} {path_to_dex} -o {build_apk}.jar"


def escape_windows_path(path):
    if "\\" in path and OS == "Windows":
        try:
            path = path.encode('string-escape')
        except Exception:
            path = path.encode('unicode-escape')

    return path


class Decompiler(object):
    """This class handles unpacking and decompiling APKs into Java source code.

    New compilers can be added by adding the command to run the compiler to the `DECOMPILER_COMMANDS` dictionary and
    adding them into the `Decompiler.decompilers` dictionary with the path to the decompiler on the system.
    Each decompiler should have at most the following keys in their command string:
    {path_to_decompiler}, {jar}, {build_directory} -- Keys can be less than these but there can be no extra keys.
    """
    def __init__(self, path_to_source, build_directory=None):
        """
        :param path_to_source: Path to directory, APK, or a .java file
        :param build_directory: directory to unpack and decompile APK to.
                                If directory does not exist it will be created, defaults to same directory as APK/qark
        """
        if not os.path.exists(path_to_source):
            raise ValueError("Invalid path, path must be to an APK, directory, or a Java file")

        self.path_to_source = path_to_source
        self.build_directory = os.path.join(build_directory, "qark") if build_directory else os.path.join(os.path.dirname(os.path.abspath(path_to_source)), "qark")

        # validate we are running on an APK, Directory, or Java source code
        if os.path.isfile(self.path_to_source) and os.path.splitext(self.path_to_source.lower())[1] not in (".java",
                                                                                                            ".apk"):
            raise ValueError("Invalid path, path must be to an APK, directory, or a Java file")

        if os.path.isdir(path_to_source) or is_java_file(path_to_source):
            self.source_code = True
            self.manifest_path = None
            log.debug("Decompiler got directory to run on, assuming Java source code")
            return

        self.source_code = False
        self.apk_name = os.path.splitext(os.path.basename(path_to_source))[0]  # name of APK without the .apk extension

        self.dex_path = self._unpack_apk()
        self.jar_path = self._run_dex2jar()
        self.manifest_path = self.run_apktool()

        self.decompilers = DECOMPILERS

    def run(self):
        """Top-level function which runs each decompiler and waits for them to finish decompilation."""
        if self.source_code:
            return

        decompiler_pool = ThreadPool(len(self.decompilers))

        for decompiler in self.decompilers:
            if not os.path.isdir(os.path.join(self.build_directory, decompiler.name)):
                os.makedirs(os.path.join(self.build_directory, decompiler.name))
            log.debug("Starting decompilation with %s", decompiler.name)
            decompiler_pool.apply_async(self._decompiler_function, args=(decompiler,))

        decompiler_pool.close()
        decompiler_pool.join()

        jar_name = os.path.split(self.jar_path)[-1]
        unpack_fernflower_jar(self.build_directory, jar_name)

    def _decompiler_function(self, decompiler):
        """
        Abstract function that subprocesses a call to a decompiler and passes it the path of the decompiler supplied.
        If the `self.jar_path` is not set this will run `self._run_dex2jar` to set its value.

        :param decompiler: Decompiler that extends `ExternalDecompiler` base class
        """
        if not self.jar_path:
            log.debug(".jar file path not found, trying to create dex file.")
            self.jar_path = self._run_dex2jar()

        decompiler_command = escape_windows_path(
            decompiler.command.format(path_to_decompiler=decompiler.path_to_decompiler,
                                      jar=self.jar_path,
                                      build_directory=self.build_directory))

        try:
            retcode = subprocess.call(shlex.split(decompiler_command))
        except Exception:
            log.exception("%s failed to finish decompiling, continuing", decompiler.name)
        else:
            if retcode != 0:
                log.info("Error running %s", decompiler.name)
                log.debug("Decompiler failed with command %s", decompiler_command)

    def run_apktool(self):
        """
        Runs `APK_TOOL_COMMAND` with the users path to APK to decompile the APK.
        Sets `self.manifest_path` to its correct location.
        """
        # check that java is version 1.7 or higher
        java_version = get_java_version()

        try:
            if float(java_version[:3]) < 1.7:
                raise SystemExit("Java version of 1.7 or higher is required to run apktool")
        except TypeError:
            log.info("Java version %s not expected, continuing run", java_version)

        user_os = platform.system().lower()
        if user_os not in ("linux", "windows", "darwin"):
            raise SystemExit("OS %s is not supported, please use Linux, Windows, or Mac OSX", user_os)

        configure_apktool()

        custom_apktool_command = escape_windows_path(APK_TOOL_COMMAND.format(apktool_path=APK_TOOL_PATH,
                                                                             path_to_source=self.path_to_source,
                                                                             build_directory=os.path.join(
                                                                                 self.build_directory, "apktool")))
        log.debug("Calling APKTool with following command")
        log.debug(custom_apktool_command)
        try:
            subprocess.call(shlex.split(custom_apktool_command))
        except Exception:
            log.exception("Failed to run APKTool with command: %s", custom_apktool_command)
            raise SystemExit("Failed to run APKTool")

        log.debug("APKTool finish executing, trying to move manifest into proper location")
        # copy valid XML file to correct location
        shutil.move(os.path.join(self.build_directory, "apktool", "AndroidManifest.xml"),
                    os.path.join(self.build_directory, "AndroidManifest.xml"))
        log.debug("Manifest moved successfully")

        log.debug("Removing apktool subdirectory of build")
        # remove the apktool generated files (only needed manifest file)
        shutil.rmtree(os.path.join(self.build_directory, "apktool"))
        log.debug("Removed apktool directory")

        return os.path.join(self.build_directory, "AndroidManifest.xml")

    def _unpack_apk(self):
        """
        Unpacks an APK and extracts its contents.

        :return: location for `self.dex_path` to use
        :rtype: os.path object
        """
        log.debug("Unpacking apk")
        unzip_file(self.path_to_source, destination_to_unzip=self.build_directory)

        return os.path.join(self.build_directory, "classes.dex")

    def _run_dex2jar(self):
        """Runs dex2jar in the lib on the dex file.

        If `self.dex_path` is None or empty it will run `_unpack_apk` to get a value for it.
        """
        # dex_path should always be set if being called through run
        if not self.dex_path:
            log.debug("Path to .dex file not found, unpacking APK")
            self.dex_path = self._unpack_apk()

        configure_dex2jar()

        dex2jar_command = escape_windows_path(DEX2JAR_COMMAND.format(dex2jar_path=os.path.join(DEX2JAR_PATH,
                                                                                               "d2j-dex2jar.{extension}".format(
                                                                                                   extension=DEX2JAR_EXTENSION)),
                                                                     path_to_dex=self.dex_path,
                                                                     build_apk=os.path.join(self.build_directory,
                                                                                            self.apk_name)))

        log.debug("Running dex2jar with command %s", dex2jar_command)
        try:
            ret_code = subprocess.call(shlex.split(dex2jar_command))
            if ret_code != 0:
                log.critical("Error running dex2jar command: %s", dex2jar_command)
                raise SystemExit("Error running dex2jar")
        except Exception:
            log.exception("Error running dex2jar command: %s", dex2jar_command)
            raise SystemExit("Error running dex2jar command")

        return os.path.join(self.build_directory, "{apk_name}.jar".format(apk_name=self.apk_name))


def unzip_file(file_to_unzip, destination_to_unzip="unzip_apk"):
    """
    Extract all directories in the zip to the destination.

    :param str file_to_unzip:
    :param str destination_to_unzip:
    """
    if not os.path.isdir(destination_to_unzip):
        os.makedirs(destination_to_unzip)
    try:
        zipped_apk = zipfile.ZipFile(file_to_unzip, "r")
        zipped_apk.extractall(path=destination_to_unzip)
    except Exception:
        log.exception("Failed to extract zipped APK from %s to %s", file_to_unzip, destination_to_unzip)
        raise SystemExit("Failed to extract zipped APK")


def get_java_version():
    """
    Run `java -version` and return its output

    :return: String of the version like '1.7.0' or '1.6.0_36'
    :rtype: str
    :raise: Exception if `java -version` fails to run properly
    """
    log.debug("Getting java version")
    try:
        full_version = subprocess.check_output(["java", "-version"], stderr=subprocess.STDOUT).decode('utf-8')
    except Exception:
        raise SystemExit("Error getting java version, is Java installed?")

    log.debug("Got full java version %s", full_version)
    # use regex to search for version like "1.6.0_38"
    version_regex = re.search(JAVA_VERSION_REGEX, full_version)

    try:
        return version_regex.group(1)
    except TypeError:
        raise SystemExit("Java version %s doesn't match regex search of %s", full_version, JAVA_VERSION_REGEX)


def configure_apktool():
    """Attempts to make apktool in lib/apktool/{operating_system}/ executable."""
    try:
        make_executable(os.path.join(APK_TOOL_PATH, "apktool.jar"))
    except Exception:
        log.exception("Failed to make APKTool files executable, continuing")


def make_executable(file_path):
    """
    Make the file at `file_path` executable.

    :param file_path: path to file to make executable
    """
    file_path = escape_windows_path(file_path)

    try:
        st = os.stat(file_path)
        os.chmod(file_path, st.st_mode | stat.S_IEXEC)
    except Exception:
        log.exception("Failed to make files executable, continuing")
        raise


def configure_dex2jar():
    make_executable(os.path.join(DEX2JAR_PATH, DEX2JAR_EXECUTABLE))
    make_executable(os.path.join(DEX2JAR_PATH, DEX2JAR_INVOKE))


def unpack_fernflower_jar(build_directory, jar_name):
    """Fernflower puts its decompiled code back into a jar so we need to unpack it."""
    previous_dir = os.getcwd()
    command = ["jar", "xf", jar_name]

    try:
        os.chdir(os.path.join(build_directory, "fernflower"))
        retcode = subprocess.call(command)
    except Exception:
        log.exception("Failed to extract fernflower jar with command '%s'", " ".join(command))
    else:
        if retcode != 0:
            log.error("Failed to extract fernflower jar with command '%s'", " ".join(command))
    finally:
        os.chdir(previous_dir)



import logging

from javalang.tree import MethodInvocation

from qark.issue import Issue, Severity
from qark.plugins.webview.helpers import valid_set_method_bool
from qark.scanner.plugin import JavaASTPlugin

log = logging.getLogger(__name__)

SET_DOM_STORAGE_ENABLED_DESCRIPTION = (
    "DOM Storage enabled for this WebView, there is a potential for caching sensitive information."
)


class SetDomStorageEnabled(JavaASTPlugin):
    """This plugin checks if the `setDomStorageEnabled` method is called with a value of `true`."""
    def __init__(self):
        super(SetDomStorageEnabled, self).__init__(category="webview", name="Webview enables DOM Storage",
                                                   description=SET_DOM_STORAGE_ENABLED_DESCRIPTION)
        self.severity = Severity.WARNING
        self.java_method_name = "setDomStorageEnabled"

    def run(self):
        for _, method_invocation in self.java_ast.filter(MethodInvocation):
            if valid_set_method_bool(method_invocation, str_bool="true", method_name=self.java_method_name):
                self.issues.append(Issue(category=self.category, name=self.name, severity=self.severity,
                                         description=self.description, line_number=method_invocation.position,
                                         file_object=self.file_path))


plugin = SetDomStorageEnabled()



import logging
import re

from qark.issue import Severity, Issue
from qark.scanner.plugin import JavaASTPlugin

log = logging.getLogger(__name__)

CHECK_PERMISSIONS_DESCRIPTION = (
    "Be careful with use of {used_permission} permission function\nApp maybe vulnerable to Privilege escalation or "
    "Confused Deputy Attack. This function can grant access to malicious application, lacking the "
    "appropriate permission, by assuming your applications permissions. This means a malicious application, "
    "without appropriate permissions, can bypass its permission check by using your application"
    "permission to get access to otherwise denied resources. Use - {recommended_permission}CallingPermission instead. "
    "Reference: https://developer.android.com/reference/android/content/Context.html\n"
)
CHECK_PERMISSION_REGEX = re.compile(r'checkCallingOrSelfPermission|checkCallingOrSelfUriPermission|checkPermission')
ENFORCE_PERMISSION_REGEX = re.compile(
    'enforceCallingOrSelfPermission|enforceCallingOrSelfUriPermission|enforcePermission')


class CheckPermissions(JavaASTPlugin):
    def __init__(self):
        super(CheckPermissions, self).__init__(category="manifest",
                                               name="Potientially vulnerable check permission function called",
                                               description=CHECK_PERMISSIONS_DESCRIPTION)
        self.severity = Severity.WARNING

    def run(self):
        if any("Context" in imp.path for imp in self.java_ast.imports):
            if re.search(CHECK_PERMISSION_REGEX, self.file_contents):
                self.issues.append(Issue(
                    category=self.category, severity=self.severity, name=self.name,
                    description=self.description.format(used_permission="Check", recommended_permission="check"),
                    file_object=self.file_path)
                )
            if re.search(ENFORCE_PERMISSION_REGEX, self.file_contents):
                self.issues.append(Issue(
                    category=self.category, severity=self.severity, name=self.name,
                    description=self.description.format(used_permission="Enforce", recommended_permission="enforce"),
                    file_object=self.file_path)
                )


plugin = CheckPermissions()



import os
import logging

import re
import javalang

from qark.scanner.plugin import JavaASTPlugin
from qark.issue import Severity, Issue

log = logging.getLogger(__name__)


class RSACipherCheck(JavaASTPlugin):
    def __init__(self):

        super(RSACipherCheck, self).__init__(category="crypto", name="RSA Cipher Usage",
                                             description="RSA mode without padding is an insecure encryption technique and prone to data leakage")

        self.severity = Severity.VULNERABILITY

    def run(self):
        method_invocations = self.java_ast.filter(javalang.tree.MethodInvocation)
        for _, method_invocation_node in method_invocations:
            try:
                method_name = method_invocation_node.member
                encryption_type = method_invocation_node.arguments[0].value
                qualifier = method_invocation_node.qualifier  # the thing that getInstance is called on
                if method_name == 'getInstance' and qualifier == 'Cipher' and re.search(r'RSA/.+/NoPadding',
                                                                                        encryption_type):
                    description = "getInstance should not be called with RSA without padding as the cipher mode, as it is insecure."
                    self.issues.append(
                        Issue(self.category, self.name, self.severity, description, file_object=self.file_path))
            except Exception:
                continue


plugin = RSACipherCheck()



"""
This plugin determines if the following methods are called:

1. getExternalFilesDir
2. getExternalFilesDirs
3. getExternalMediaDirs
4. getExternalStoragePublicDirectory

"""

import logging

import javalang
from javalang.tree import MethodInvocation

from qark.issue import Severity, Issue
from qark.plugins.helpers import java_files_from_files
from qark.scanner.plugin import JavaASTPlugin

log = logging.getLogger(__name__)

EXTERNAL_STORAGE_DESCRIPTION = (
    "Reading files stored on {storage_location} makes it vulnerable to data injection attacks. "
    "Note that this code does no error checking and there is no security enforced with these files. "
    "For example, any application holding WRITE_EXTERNAL_STORAGE can write to these files. "
    "Reference: https://developer.android.com/reference/android/content/Context.html"
)

EXTERNAL_FILES_DIR_METHOD = 'getExternalFilesDir'
EXTERNAL_FILES_DIRS_METHOD = 'getExternalFilesDirs'
EXTERNAL_MEDIA_DIR_METHOD = 'getExternalMediaDirs'
EXTERNAL_STORAGE_PUBLIC_DIR_METHOD = 'getExternalStoragePublicDirectory'


class ExternalStorage(JavaASTPlugin):
    def __init__(self):
        super(ExternalStorage, self).__init__(category="file", name="External storage used",
                                              description=EXTERNAL_STORAGE_DESCRIPTION)
        self.severity = Severity.WARNING

    def run(self):
        for _, method_invocation in self.java_ast.filter(MethodInvocation):
            storage_location = None
            if (method_invocation.member == EXTERNAL_FILES_DIR_METHOD
                    or method_invocation.member == EXTERNAL_FILES_DIRS_METHOD):
                storage_location = "External Storage"

            elif method_invocation.member == EXTERNAL_MEDIA_DIR_METHOD:
                storage_location = "External Media Directory"

            elif method_invocation.member == EXTERNAL_STORAGE_PUBLIC_DIR_METHOD:
                storage_location = "External Storage Public Directory"

            if storage_location:
                self.issues.append(Issue(
                    category=self.category, severity=self.severity, name=self.name,
                    description=self.description,
                    file_object=self.file_path,
                    line_number=method_invocation.position)
                )


plugin = ExternalStorage()



import logging
import re

from qark.issue import Severity, Issue
from qark.scanner.plugin import JavaASTPlugin

log = logging.getLogger(__name__)

TASK_AFFINITY_DESCRIPTION = (
    "FLAG_ACTIVITY_{type}_TASK - intent flag is set. This results in activity being loaded as a part of a new task. "
    "This can be abused in the task hijacking attack. "
    "Reference: https://www.usenix.org/system/files/conference/usenixsecurity15/sec15-paper-ren-chuangang.pdf"
)
NEW_TASK_REGEX = re.compile(r'FLAG_ACTIVITY_NEW_TASK')
MULTIPLE_TASK_REGEX = re.compile(r'FLAG_ACTIVITY_MULTIPLE_TASK')


class TaskAffinity(JavaASTPlugin):
    def __init__(self):
        super(TaskAffinity, self).__init__(category="generic", name="Potential task hijacking",
                                           description=TASK_AFFINITY_DESCRIPTION)
        self.severity = Severity.INFO

    def run(self):
        if any("Intent" in import_decl.path for import_decl in self.java_ast.imports):
            description = None

            if re.search(NEW_TASK_REGEX, self.file_contents):
                description = TASK_AFFINITY_DESCRIPTION.format("NEW")
            elif re.search(MULTIPLE_TASK_REGEX, self.file_contents):
                description = TASK_AFFINITY_DESCRIPTION.format("MULTIPLE")

            if description:
                self.issues.append(Issue(category=self.category,
                                         severity=self.severity,
                                         name=self.name,
                                         description=description,
                                         file_object=self.file_path))


plugin = TaskAffinity()



import logging

from qark.issue import Severity
from qark.plugins.webview.helpers import webview_default_vulnerable
from qark.scanner.plugin import JavaASTPlugin

log = logging.getLogger(__name__)

SET_ALLOW_CONTENT_ACCESS_DESCRIPTION = (
    "While not a vulnerability by itself, it appears this app does not explicitly disable Content Provider access "
    "from WebViews. If the WebViews take in untrusted input, this can allow for data theft. To validate this "
    "vulnerability, load the following local file in this WebView: "
    "file://qark/poc/html/WV_CPA_WARNING.html"
)


class SetAllowContentAccess(JavaASTPlugin):
    """This plugin checks if the webview calls setAllowContentAccess(false), otherwise the webview is vulnerable
    (defaults to true)."""
    def __init__(self):
        super(SetAllowContentAccess, self).__init__(category="webview", name="Webview enables content access",
                                                    description=SET_ALLOW_CONTENT_ACCESS_DESCRIPTION)
        self.severity = Severity.WARNING

    def run(self):
        self.issues.extend(webview_default_vulnerable(self.java_ast, method_name="setAllowContentAccess",
                                                      issue_name=self.name,
                                                      description=self.description, file_object=self.file_path,
                                                      severity=self.severity))


plugin = SetAllowContentAccess()



import logging

from javalang.tree import MethodInvocation

from qark.issue import Issue, Severity
from qark.plugins.helpers import valid_method_invocation
from qark.scanner.plugin import JavaASTPlugin, ManifestPlugin

log = logging.getLogger(__name__)

ADD_JAVASCRIPT_INTERFACE_DESCRIPTION = (
    "This webview uses the addJavascriptInterface method in a pre-API 17 app, which exposes all public methods to "
    "Javascript running in the WebView. If this webview loads untrusted content or trusted content over plain-text "
    "HTTP, this represents a MAJOR issue! Reference: "
    "https://labs.mwrinfosecurity.com/blog/2013/09/24/webview-addjavascriptinterface-remote-code-execution/. "
    "To validate this vulnerability, load the following local file in this WebView: "
    "file://qark/poc/html/BAD_JS_INT.html"
)


class AddJavascriptInterface(JavaASTPlugin, ManifestPlugin):
    """This plugin checks if the `addJavaScriptInterface` method is called with a min_sdk of below 17."""
    def __init__(self):
        super(AddJavascriptInterface, self).__init__(category="webview",
                                                     name="Webview uses addJavascriptInterface pre-API 17",
                                                     description=ADD_JAVASCRIPT_INTERFACE_DESCRIPTION)
        self.severity = Severity.WARNING
        self.java_method_name = "addJavascriptInterface"

    def run(self):
        if self.min_sdk <= 16:
            for _, method_invocation in self.java_ast.filter(MethodInvocation):
                if valid_method_invocation(method_invocation, method_name=self.java_method_name, num_arguments=2):
                    self.issues.append(Issue(category=self.category, name=self.name, severity=self.severity,
                                             description=self.description, line_number=method_invocation.position,
                                             file_object=self.file_path))


plugin = AddJavascriptInterface()



import logging
import re

from qark.issue import Severity, Issue
from qark.utils import is_java_file
from qark.scanner.plugin import FileContentsPlugin

log = logging.getLogger(__name__)

HARDCODED_HTTP_DESCRIPTION = (
    "Application contains hardcoded HTTP url: {http_url}, unless HSTS is implemented, this request can be "
    "intercepted and modified by a man-in-the-middle attack."
)

HTTP_URL_REGEX = re.compile(r'http://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*(),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+')


class HardcodedHTTP(FileContentsPlugin):
    def __init__(self):
        super(HardcodedHTTP, self).__init__(category="file", name="Hardcoded HTTP url found",
                                            description=HARDCODED_HTTP_DESCRIPTION)
        self.severity = Severity.INFO

    def run(self):
        if not is_java_file(self.file_path):
            return

        for line_number, line in enumerate(self.file_contents.split('\n')):
            http_url_match = re.search(HTTP_URL_REGEX, line)
            if http_url_match:
                self.issues.append(Issue(
                    category=self.category, severity=self.severity, name=self.name,
                    description=self.description.format(http_url=http_url_match.group(0)),
                    file_object=self.file_path,
                    line_number=(line_number, 0))
                )


plugin = HardcodedHTTP()



import logging

from qark.issue import Issue, Severity
from qark.plugins.manifest_helpers import get_min_sdk
from qark.scanner.plugin import ManifestPlugin

log = logging.getLogger(__name__)

TAP_JACKING = ("Since the minSdkVersion is less that 9, it is likely this application is vulnerable to TapJacking. "
               "QARK made no attempt to confirm, as the protection would have to be custom code, which is difficult for"
               " QARK to examine and understand properly. This vulnerability allows a malicious application to lay on "
               "top of this app, while letting the key strokes pass through to the application below. This can cause "
               "users to take unwanted actions, within the victim application, similar to Clickjacking on websites. "
               "Please select the appropriate options in the exploitation menus to verify manually using QARK's "
               "exploit APK. Note: The QARK proof-of-concept is transparent, but in real-world attacks, it would "
               "likely not be. This is done solely to aid in testing. For more information: "
               "https://media.blackhat.com/ad-12/Niemietz/bh-ad-12-androidmarcus_niemietz-WP.pdf")


class MinSDK(ManifestPlugin):
    """This plugin will create issues depending on the manifest's minimum SDK. For instance
    tapjacking is only protected when min_sdk > 9 (or custom code is used)."""
    def __init__(self):
        super(MinSDK, self).__init__(name="MinSDK checks", category="manifest")

        self.severity = Severity.WARNING

    def run(self):
        if self.min_sdk < 9:
            self.issues.append(Issue(category=self.category, name="Tap Jacking possible",
                                     severity=Severity.VULNERABILITY, description=TAP_JACKING))


plugin = MinSDK()



import os
import logging

import javalang

from qark.scanner.plugin import JavaASTPlugin
from qark.issue import Severity, Issue

log = logging.getLogger(__name__)


class SeedWithSecureRandom(JavaASTPlugin):

    INSECURE_FUNCTIONS = ("setSeed", "generateSeed")

    def __init__(self):

        super(SeedWithSecureRandom, self).__init__(category="crypto",
                                                   name="Random number generator is seeded with SecureSeed",
                                                   description=(
                                                       "Specifying a fixed seed will cause a predictable sequence of numbers. "
                                                       "This may be useful for testing, but not for secure use"))

        self.severity = Severity.WARNING

    def _imports_secure_seed(self, tree):
        """Checks if a tree imports java.security.SecureRandom, and returns True if the import exists"""
        return any(imp.path == "java.security.SecureRandom" for imp in tree.imports)

    def run(self):
        if not self._imports_secure_seed(self.java_ast):  # doesn't import the insecure function
            return

        method_invocations = self.java_ast.filter(javalang.tree.MethodInvocation)
        for _, method_invocation_node in method_invocations:
            if method_invocation_node.member in SeedWithSecureRandom.INSECURE_FUNCTIONS:
                self.issues.append(Issue(self.category, self.name, self.severity, self.description,
                                         file_object=self.file_path))


plugin = SeedWithSecureRandom()



import abc
import os

LIB_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", "lib")
PATH_TO_DECOMPILERS = os.path.join(LIB_PATH, "decompilers")


class ExternalDecompiler(object):
    __meta__ = abc.ABCMeta

    def __init__(self, name, path_to_decompiler, command=None):
        self.name = name.lower()
        self.path_to_decompiler = path_to_decompiler
        self.command = command


class CFR(ExternalDecompiler):
    def __init__(self):
        ExternalDecompiler.__init__(self,
                                    name="cfr",
                                    path_to_decompiler=os.path.join(PATH_TO_DECOMPILERS, "cfr_0_124.jar"),
                                    command="java -jar {path_to_decompiler} {jar} --outputdir {build_directory}/cfr")


class Procyon(ExternalDecompiler):
    def __init__(self):
        ExternalDecompiler.__init__(self,
                                    name="procyon",
                                    path_to_decompiler=os.path.join(PATH_TO_DECOMPILERS,
                                                                    "procyon-decompiler-1.0.jar"),
                                    command="java -jar {path_to_decompiler} {jar} -o {build_directory}/procyon")


class Fernflower(ExternalDecompiler):
    def __init__(self):
        ExternalDecompiler.__init__(self,
                                    name="fernflower",
                                    path_to_decompiler=os.path.join(PATH_TO_DECOMPILERS,
                                                                    "fernflower.jar"),
                                    command="java -jar {path_to_decompiler} -ren=1 {jar} {build_directory}/fernflower")


DECOMPILERS = (CFR(), Procyon(), Fernflower())



import logging

import javalang
from javalang.tree import ClassCreator, MemberReference, MethodInvocation

from qark.issue import Issue, Severity
from qark.scanner.plugin import JavaASTPlugin

log = logging.getLogger(__name__)

ALLOW_ALL_HOSTNAME_VERIFIER_DESC = ("This can allow for impromper x.509 certificate validation wherein the DNS "
                                    "hostname does not match the Common or Subject Alternative Name(s) on the "
                                    "certificate, making the application vulnerable to Man-In-The-Middle attacks. "
                                    "This means the application may potentially accept a certificate from any trusted "
                                    "CA, regardless of the domain it was issued for. The can be validated using the "
                                    "free version of Burpsuite by installing the Portswigger CA certificate, thereby "
                                    "making it a trusted CA on the device. Set the device network settings to use the "
                                    "Burpsuite proxy, then go Proxy > Options > Edit the Proxy Listener by changing "
                                    "the Certificate tab to Generate a CA-signed certificate with a specific hostname "
                                    "and enter a domain like foobar.com which doesn't match the domain name(s) the app "
                                    "is connecting to normally. You should always verify your results by visiting an "
                                    "https site in the native browser and confirming you see a certificate warning. "
                                    "For details, please see: "
                                    "https://developer.android.com/training/articles/security-ssl.html")


class HostnameVerifier(JavaASTPlugin):
    """
    This plugin checks if:
     1. `AllowAllHostnameVerifier` is instantiated
     2. `setHostnameVerifier` is called with a value of `.ALLOW_ALL_HOSTNAME_VERIFIERS`
    """
    def __init__(self):
        super(HostnameVerifier, self).__init__(category="cert", name="Hostname Verifier")
        self.severity = Severity.WARNING

    def run(self):
        self._allow_all_hostname_verifier_created(self.java_ast, self.file_path)
        self._set_hostname_verifier_allow_all(self.java_ast, self.file_path)
        self._null_hostname_verifier(self.java_ast, self.file_path)

    def _allow_all_hostname_verifier_created(self, tree, current_file):
        """
        Checks for a class creation of `AllowAllHostnameVerifier`.

        :param tree: javalang parsed tree
        """
        hostname_verifiers = [hostname_verifier for _, hostname_verifier in tree.filter(ClassCreator)
                              if hostname_verifier.type.name == "AllowAllHostnameVerifier"]
        if hostname_verifiers:
            self.issues.append(Issue(category=self.category, name="Allow all hostname verifier used",
                                     severity=Severity.WARNING, description=ALLOW_ALL_HOSTNAME_VERIFIER_DESC,
                                     file_object=current_file))

    def _set_hostname_verifier_allow_all(self, tree, current_file):
        """Check for setHostnameVerifier with argument ALLOW_ALL_HOSTNAME_VERIFIER

        :param tree: javalang parsed tree
        """
        set_hostname_verifiers = (allow_all_verifier for _, allow_all_verifier in tree.filter(MethodInvocation)
                                  if allow_all_verifier.member == "setHostnameVerifier"
                                  and len(allow_all_verifier.arguments) == 1
                                  and type(allow_all_verifier.arguments[0]) is MemberReference
                                  and allow_all_verifier.arguments[0].member == "ALLOW_ALL_HOSTNAME_VERIFIER")
        for set_hostname_verifier in set_hostname_verifiers:
            self.issues.append(Issue(category=self.category, name="setHostnameVerifier set to ALLOW_ALL",
                                     severity=Severity.WARNING, description=ALLOW_ALL_HOSTNAME_VERIFIER_DESC,
                                     file_object=current_file, line_number=set_hostname_verifier.position))

    def _null_hostname_verifier(self, tree, current_file):
        """
        Checks for a class creation of `NullHostNameVerifier` or `NullHostnameVerifier`.

        :param tree: javalang parsed tree
        """
        hostname_verifiers = [hostname_verifier for _, hostname_verifier in tree.filter(ClassCreator)
                              if hostname_verifier.type.name == "NullHostNameVerifier"
                              or hostname_verifier.type.name == "NullHostnameVerifier"]
        if hostname_verifiers:
            self.issues.append(Issue(category=self.category, name="Allow all hostname verifier used",
                                     severity=Severity.WARNING, description=ALLOW_ALL_HOSTNAME_VERIFIER_DESC,
                                     file_object=current_file))


plugin = HostnameVerifier()



from __future__ import absolute_import

import logging
from os import (
    walk,
    path
)

from qark.scanner.plugin import PluginObserver
from qark.scanner.plugin import get_plugin_source, get_plugins
from qark.scanner.plugin import ManifestPlugin
from qark.plugins.manifest.exported_tags import ExportedTags
from qark.xml_helpers import get_manifest_out_of_files
from qark.utils import is_java_file

log = logging.getLogger(__name__)

PLUGIN_CATEGORIES = ("manifest", "broadcast", "file", "crypto", "intent", "cert", "webview", "generic")


class Scanner(object):

    def __init__(self, manifest_path, path_to_source):
        """
        Creates the scanner.

        :param str manifest_path: the path to the manifest file
        :param str path_to_source: the path to the source code
        """
        self.files = set()
        self.issues = []
        self.manifest_path = manifest_path

        self.path_to_source = path_to_source

        self._gather_files()

    def run(self):
        """
        Runs all the plugin checks by category.
        """
        plugins = []
        for category in PLUGIN_CATEGORIES:
            plugin_source = get_plugin_source(category=category)

            if category == "manifest":
                # Manifest plugins only need to run once, so we run them and continue
                manifest_plugins = get_plugins(category)
                ManifestPlugin.update_manifest(self.manifest_path)
                if ManifestPlugin.manifest_xml is not None:

                    for plugin in [plugin_source.load_plugin(plugin_name).plugin for plugin_name in manifest_plugins]:
                        # Give more detail to the ExportedTags manifest plugin as it is important for building the exploit
                        #   APK. Careful!
                        plugin.all_files = self.files

                        plugin.run()
                        self.issues.extend(plugin.issues)
                    continue

            for plugin_name in get_plugins(category):
                plugins.append(plugin_source.load_plugin(plugin_name).plugin)

        self._run_checks(plugins)

    def _run_checks(self, plugins):
        """Run all the plugins (besides manifest) on every file."""
        current_file_subject = Subject()
        plugins = list(observer_plugin for observer_plugin in plugins if isinstance(observer_plugin, PluginObserver))
        for plugin in plugins:
            current_file_subject.register(plugin)

        for filepath in self.files:
            current_file_subject.notify(filepath)  # All the plugins will be running now
            # reset the plugin file data to None
            current_file_subject.reset()

        for plugin in plugins:
            self.issues.extend(plugin.issues)

        #     ast = None
        #
        #     try:
        #         with open(filepath, 'r') as f:
        #             file_contents = f.read()
        #     except Exception:
        #         log.exception("Unable to read file %s", filepath)
        #         file_contents = None
        #
        #     if file_contents and is_java_file(filepath):
        #         try:
        #             ast = javalang.parse.parse(file_contents)
        #         except (javalang.parser.JavaSyntaxError, IndexError):
        #             log.debug("Unable to parse AST for file %s", filepath)
        #
        #     for plugin in plugins:
        #         log.debug("Running plugin %s", plugin.name)
        #         plugin.run(filepath,
        #                    apk_constants=self.apk_constants,
        #                    java_ast=ast,
        #                    file_contents=file_contents,
        #                    all_files=self.files)
        #
        # for plugin in plugins:
        #     self.issues.extend(plugin.issues)

    def _gather_files(self):
        """Walks the `path_to_source` and updates the `self.files` set with new files."""
        if is_java_file(self.path_to_source):
            self.files.add(self.path_to_source)
            log.debug("Added single java file to scanner")
            return

        log.debug("Adding files to scanner...")
        try:
            for (dir_path, _, file_names) in walk(self.path_to_source):
                for file_name in file_names:
                    self.files.add(path.join(dir_path, file_name))
        except AttributeError:
            log.debug("Decompiler does not have a build directory")


class Subject(object):
    """"""

    def __init__(self):
        self.observers = []

    def register(self, observer):
        """
        :param PluginObserver observer:
        """
        self.observers.append(observer)

    def unregister(self, observer):
        """
        :param PluginObserver observer:
        """
        self.observers.remove(observer)

    def notify(self, file_path):
        for observer in self.observers:
            observer.update(file_path, call_run=True)

    def reset(self):
        for observer in self.observers:
            observer.reset()



from __future__ import absolute_import

from os import path

from jinja2 import Environment, PackageLoader, select_autoescape, Template

from qark.issue import (Issue, Severity, issue_json)  # noqa:F401 These are expected to be used later.
from qark.utils import create_directories_to_path

DEFAULT_REPORT_PATH = path.join(path.dirname(path.realpath(__file__)), 'report', '')


jinja_env = Environment(
    loader=PackageLoader('qark', 'templates'),
    autoescape=select_autoescape(['html', 'xml'])
)

jinja_env.filters['issue_json'] = issue_json


class Report(object):
    """An object to store issues against and to generate reports in different formats.

    There is one instance created per QARK run and it uses a classic Singleton pattern
    to make it easy to get a reference to that instance anywhere in QARK.
    """

    # The one instance to rule them all
    # http://python-3-patterns-idioms-test.readthedocs.io/en/latest/Singleton.html#the-singleton
    __instance = None

    def __new__(cls, issues=None, report_path=None):
        if Report.__instance is None:
            Report.__instance = object.__new__(cls)

        return Report.__instance

    def __init__(self, issues=None, report_path=None):
        """This will give you an instance of a report, with a default report path which is local
        to where QARK is on the file system.

        :param report_path: The path to the report directory where all generated report files will be written.
        :type report_path: str or None

        """
        self.issues = issues if issues else []
        self.report_path = report_path or DEFAULT_REPORT_PATH

    def generate(self, file_type='html', template_file=None):
        """This method uses Jinja2 to generate a standalone HTML version of the report.

        :param str file_type:     The type of file for the report. Defaults to 'html'.
        :param str template_file: The path to an optional template file to override the default.
        :return: Path to the written report
        :rtype: str
        """
        create_directories_to_path(self.report_path)

        full_report_path = path.join(self.report_path, 'report.{file_type}'.format(file_type=file_type))

        with open(full_report_path, mode='w') as report_file:
            if not template_file:
                template = jinja_env.get_template('{file_type}_report.jinja'.format(file_type=file_type))
            else:
                template = Template(template_file)
            report_file.write(template.render(issues=list(self.issues)))

        return full_report_path



import logging

from qark.issue import Severity, Issue
from qark.scanner.plugin import ManifestPlugin

log = logging.getLogger(__name__)

TASK_REPARENTING_DESCRIPTION = (
    "This allows an existing activity to be reparented to a new native task i.e task having the same affinity as the "
    "activity. This may lead to UI spoofing attack on this application."
    "https://www.usenix.org/system/files/conference/usenixsecurity15/sec15-paper-ren-chuangang.pdf"
)


class TaskReparenting(ManifestPlugin):
    def __init__(self):
        super(TaskReparenting, self).__init__(category="manifest", name="android:allowTaskReparenting='true' found",
                                              description=TASK_REPARENTING_DESCRIPTION)
        self.severity = Severity.WARNING

    def run(self):
        with open(self.manifest_path, "r") as manifest_file:
            for line_number, line in enumerate(manifest_file):
                if 'android:allowTaskReparenting="true"' in line:
                    self.issues.append(Issue(
                        category=self.category, severity=self.severity, name=self.name,
                        description=self.description,
                        file_object=self.manifest_path,
                        line_number=line_number)
                    )


plugin = TaskReparenting()



import logging
import os
import re
import shutil

from javalang.tree import MethodInvocation
from qark.plugins.manifest_helpers import get_min_sdk
from qark.utils import is_java_file

log = logging.getLogger(__name__)

EXCLUDE_REGEXES = (r'^\s*(//|/\*)',
                   r'^\s*\*',
                   r'.*\*\/$',
                   r'^\s*Log\..\(',
                   r'(.*)(public|private)\s(String|List)')

EXCLUSION_REGEX = re.compile("|".join(EXCLUDE_REGEXES))


def run_regex(filename, rex):
    """
    Read a file line by line, run a regular expression against the content and return list of things that require inspection

    :param str filename: path to file
    :param rex: can be a compiled regex or a string of the regex to search
    """
    things_to_inspect = []
    try:
        with open(filename) as f:
            for curr_line in f:
                if re.search(rex, curr_line):
                    # exclude everything in EXCLUDE_REGEXES
                    if not re.match(EXCLUSION_REGEX, curr_line):
                        things_to_inspect.append(curr_line)
    except IOError:
        log.debug("Unable to open file: %s results will be inaccurate", filename)
    except UnicodeDecodeError:
        log.debug("Error reading file: %s most likely it is of an invalid type", filename)
    except Exception:
        log.exception("Failed to read file: %s", filename)
    return things_to_inspect


def java_files_from_files(files):
    """
    Returns a generator of everything in `files` that ends with the `.java` extension.

    :param list files:
    :return: generator of file paths
    """
    return (file_path for file_path in files if is_java_file(file_path))


def remove_dict_entry_by_value(dictionary, value):
    """Helper to remove an entry in a dictionary by its value instead of its key."""
    return {k: v for k, v in dictionary.items() if v != dictionary.get(value)}


def valid_method_invocation(method_invocation, method_name, num_arguments):
    """
    Determines if a `MethodInvocation` has the name `method_name` and the number of arguments `num_arguments`

    :param MethodInvocation method_invocation: the javalang MethodInvocation
    :param str method_name: the name of the method that should be called
    :param int num_arguments: the number of arguments the method should contain
    :return: Whether the method invocation matches the parameters
    :rtype: bool
    """
    return (isinstance(method_invocation, MethodInvocation)
            and method_invocation.member == method_name
            and len(method_invocation.arguments) == num_arguments)


def get_min_sdk_from_files(files, apk_constants=None):
    """
    Get the min_sdk from either the `apk_constants` if it exists, or the manifest file in `files` if it exists. If
    neither exists, return 1 as the default minimum SDK

    :param Iterable files: file names that should be iterated over
    :param Dict apk_constants: dictionary that can have constants throughout the apk
    :return: min_sdk if it exists or 1
    :rtype: int
    """
    try:
        # int conversion to change it if it is a NoneType, which will throw TypeError
        return int(apk_constants["min_sdk"])
    except (KeyError, TypeError):
        for decompiled_file in files:
            if decompiled_file.lower().endswith("{separator}androidmanifest.xml".format(separator=os.sep)):
                return get_min_sdk(decompiled_file)
    return 1


def copy_directory_to_location(directory_to_copy, destination):
    log.debug("Copying from %s to %s", directory_to_copy, destination)
    try:
        shutil.copytree(src=directory_to_copy, dst=destination)
    except Exception:
        raise



import logging
import re

from qark.issue import Severity, Issue
from qark.scanner.plugin import FileContentsPlugin

log = logging.getLogger(__name__)

PHONE_IDENTIFIER_DESCRIPTION = (
    "Access of phone number or IMEI, is detected. Avoid storing or transmitting this data."
)

TELEPHONY_MANAGER_VARIABLE_NAMES_REGEX = re.compile(r'(android\.telephony\.)?TelephonyManager\s(\w*?)([,);]|(\s=))')
TELEPHONY_MANAGER_REGEX = re.compile(r'android\.telephony\.TelephonyManager')
TELEPHONY_INLINE_REGEX = re.compile(r'\({2,}(android.telephony.)?TelephonyManager\)\w*?\.getSystemService\([\'\"]phone'
                                    r'[\'\"]\){2,}\.(getLine1Number|getDeviceId)')


class PhoneIdentifier(FileContentsPlugin):
    def __init__(self):
        super(PhoneIdentifier, self).__init__(category="file", name="Phone number or IMEI detected",
                                              description=PHONE_IDENTIFIER_DESCRIPTION)
        self.severity = Severity.INFO

    def run(self):
        if re.search(TELEPHONY_MANAGER_REGEX, self.file_contents):
            if re.search(TELEPHONY_INLINE_REGEX, self.file_contents):
                self._add_issue(java_path=self.file_path)

            else:

                for match in re.finditer(TELEPHONY_MANAGER_VARIABLE_NAMES_REGEX, self.file_contents):

                    for variable_name in match.group(2):

                        if re.search(r'{var_name}\.(getLine1Number|getDeviceId)\(.*?\)'.format(var_name=variable_name),
                                     self.file_contents):
                            self._add_issue(java_path=self.file_path)

    def _add_issue(self, java_path):
        self.issues.append(Issue(
            category=self.category, severity=self.severity, name=self.name,
            description=self.description,
            file_object=java_path)
        )


plugin = PhoneIdentifier()



from __future__ import absolute_import

import logging
import os
import shlex
import shutil
import subprocess

from six import StringIO
from six.moves import configparser

from qark.plugins.helpers import copy_directory_to_location
from qark.plugins.manifest_helpers import get_package_from_manifest
from qark.xml_helpers import write_key_value_to_string_array_xml, write_key_value_to_xml

log = logging.getLogger(__name__)


COMPONENT_ENTRIES = {"activity": ("onCreate", "onStart"),
                     "activity-alias": ("onCreate", "onStart"),
                     "receiver": ("onReceive",),
                     "service": ("onCreate", "onBind", "onStartCommand", "onHandleIntent"),
                     "provider": ("onReceive",)
                     }

EXPLOIT_APK_TEMPLATE_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), "exploit_apk")


class APKBuilder(object):
    __instance = None

    def __new__(cls, exploit_apk_path, issues, apk_name, manifest_path, sdk_path):
        if APKBuilder.__instance is None:
            APKBuilder.__instance = object.__new__(cls)

        return APKBuilder.__instance

    def __init__(self, exploit_apk_path, issues, apk_name, manifest_path, sdk_path):
        """
        Creates the APKBuilder.

        :param str exploit_apk_path: path to where the exploit apk should be built
        :param list issues: List of `Issue` found from the scanner
        :param str apk_name: name of the examined APK
        """
        self.exploit_apk_path = os.path.join(exploit_apk_path, "{apk_name}_exploit_apk".format(apk_name=apk_name))

        # need to remove directory if it exists otherwise shutil.copytree will error from helper
        if os.path.isdir(self.exploit_apk_path):
            shutil.rmtree(self.exploit_apk_path)

        # copy template exploit APK to exploit location
        try:
            copy_directory_to_location(directory_to_copy=EXPLOIT_APK_TEMPLATE_PATH, destination=self.exploit_apk_path)
        except Exception:
            log.exception("Failed to copy %s to %s", EXPLOIT_APK_TEMPLATE_PATH, self.exploit_apk_path)
            raise SystemExit("Failed to copy %s to %s", EXPLOIT_APK_TEMPLATE_PATH, self.exploit_apk_path)

        values_path = os.path.join(self.exploit_apk_path, "app", "src", "main", "res", "values")
        self.strings_xml_path = os.path.join(values_path, "strings.xml")
        self.extra_keys_xml_path = os.path.join(values_path, "extraKeys.xml")
        self.intent_ids_xml_path = os.path.join(values_path, "intentID.xml")

        self.properties_file_path = os.path.join(self.exploit_apk_path, "local.properties")
        self.sdk_path = sdk_path

        self.issues = issues
        try:
            self.package_name = get_package_from_manifest(manifest_path)
        except IOError:
            log.exception("Failed to read manifest file at %s", manifest_path)
            raise SystemExit("Failed to read manifest file at %s", manifest_path)

    def build(self):
        self._write_additional_exploits()
        self._build_apk()

    def _write_additional_exploits(self):
        for issue in self.issues:
            self._write_exported_tags(issue)

    def _write_exported_tags(self, issue):
        if issue.apk_exploit_dict:
            try:
                tag_enum = issue.apk_exploit_dict["exported_enum"]
                tag_name = issue.apk_exploit_dict["tag_name"]
                package_name = issue.apk_exploit_dict["package_name"]
            except KeyError:
                # issue is not the right type to write these values
                return

            arguments = issue.apk_exploit_dict.get("arguments")
            new_key = write_key_value_to_string_array_xml(array_name=tag_enum.parent.value,
                                                          value=tag_enum.type.value,
                                                          path=self.intent_ids_xml_path)

            # write extraKeys for tags that need it
            if arguments is not None and tag_enum.type.value in ("activity", "broadcast", "provider", "receiver"):
                for argument in arguments:
                    write_key_value_to_string_array_xml(array_name=new_key,
                                                        value=argument,
                                                        path=self.extra_keys_xml_path,
                                                        add_id=False)
            write_key_value_to_xml(key=new_key, value=package_name + tag_name, path=self.strings_xml_path)

    def _build_apk(self):
        log.debug("Building apk...")
        current_directory = os.getcwd()
        try:
            os.chdir(self.exploit_apk_path)
            write_key_value_to_xml('packageName', self.package_name, self.strings_xml_path)
            self._write_properties_file({"sdk.dir": self.sdk_path})
            command = "./gradlew assembleDebug"
            try:
                subprocess.call(shlex.split(command))
            except Exception:
                log.exception("Error running command %s")
                raise  # raise here as we can still make the report for the user
        except Exception:
            raise
        finally:
            os.chdir(current_directory)

    def _write_properties_file(self, dict_to_write, append=True):
        """
        Write each key and value into the properties file, dictionary should be simple as complex types won't
        be able to be written to properties file.

        :param dict dict_to_write: Dictionary of key=value pairs to write to properties file
        :param append: if the properties file should be appended to or not (if False, write over the file)
        """
        mode = "a" if append else "w"
        with open(self.properties_file_path, mode) as properties_file:
            for key, value in dict_to_write.items():
                properties_file.write("{key}={value}\n".format(key=key, value=value))

    def _read_properties_file(self):
        """
        Reads the local.properties file. Converts the file from .properties to a .ini by adding
        a dummy header, the header

        :return: dictionary of values in key, value pairs
        :rtype: dict
        """
        with open(self.properties_file_path, "r") as properties_file:
            config = StringIO()
            config.write('[dummy_header]\n')

            # escape percentages found in properties file (ini file they are special characters)
            config.write(properties_file.read().replace('%', '%%'))
            config.seek(0, os.SEEK_SET)

            cp = configparser.SafeConfigParser()
            cp.readfp(config)

            return dict(cp.items('dummy_section'))



from qark.scanner.plugin import JavaASTPlugin
from qark.issue import Severity, Issue

import logging
import re

from javalang.tree import MethodInvocation, ClassCreator, ReferenceType

log = logging.getLogger(__name__)


PENDING_INTENT_METHODS = ("getActivities",
                          "getService",
                          "getActivity",
                          "getBroadcast",
                          )
PENDING_INTENT_REGEX = re.compile(
    "({pending_intent_method})".format(pending_intent_method="|".join(PENDING_INTENT_METHODS)))


class ImplicitIntentToPendingIntent(JavaASTPlugin):
    """
    This plugin checks if a `new Intent` is passed into any of the `PENDING_INTENT_METHODS`.
    If there is, a Vulnerability is created.
    """
    def __init__(self):
        super(ImplicitIntentToPendingIntent, self).__init__(category="intent", name="Empty pending intent found",
                                                            description=(
                                                                "For security reasons, the Intent you supply here should almost always"
                                                                " be an explicit intent that is specify an explicit component to be delivered"
                                                                " to through Intent.setClass. A malicious application could potentially"
                                                                " intercept, redirect and/or modify this Intent. Pending Intents retain the"
                                                                " UID of your application and all related permissions, allowing another"
                                                                " application to act as yours. Reference: "
                                                                "https://developer.android.com/reference/android/app/PendingIntent.html"))
        self.severity = Severity.VULNERABILITY
        self.current_file = None

    def run(self):
        # simple search to avoid files that are not vulnerable
        if re.search("new Intent", self.file_contents) is None or re.search(PENDING_INTENT_REGEX, self.file_contents) is None:
            return

        if not any("PendingIntent" in imported_declaration.path for imported_declaration in self.java_ast.imports):
            # if PendingIntent is never imported the file is not vulnerable
            return

        self.current_file = self.file_path
        self._check_for_implicit_intents(self.java_ast)

    def _check_for_implicit_intents(self, parsed_tree):
        """
        Checks for an invocation of one of the methods in `PENDING_INTENT_METHODS` and checks to see if an implicit
        Intent is passed to it.

        :param parsed_tree: `javalang.tree.parse` object
        """
        # get all method invocations that are in PENDING_INTENT_METHODS
        pending_intent_invocations = (method_invocation for _, method_invocation
                                      in parsed_tree.filter(MethodInvocation)
                                      if method_invocation.member in PENDING_INTENT_METHODS)

        for pending_intent_invocation in pending_intent_invocations:
            # iterate over every argument in the pending intent call, looking for a "new Intent()"
            for method_argument in pending_intent_invocation.arguments:
                for _, creation in method_argument.filter(ClassCreator):
                    if len(creation.arguments) in (0, 1):  # remove any intents created with arguments
                        for _, reference_declaration in creation.filter(ReferenceType):
                            if reference_declaration.name == "Intent":
                                self.issues.append(Issue(category=self.category, severity=self.severity,
                                                         name=self.name, description=self.description,
                                                         file_object=self.current_file))


plugin = ImplicitIntentToPendingIntent()



import logging
from copy import deepcopy

from javalang.tree import MethodInvocation, MethodDeclaration, VariableDeclaration, Type, Literal

from qark.issue import Issue, Severity
from qark.plugins.helpers import valid_method_invocation, remove_dict_entry_by_value

log = logging.getLogger(__name__)


CATEGORY = "webview"


def valid_set_method_bool(method_invocation, str_bool, method_name="setAllowFileAccess"):
    """
    Determines if the `method_invocation` has the same `method_name` and has an argument of value `str_bool`

    :param MethodInvocation method_invocation: The javalang MethodInvocation
    :param str str_bool: The value of what should be in the argument
    :param str method_name: The name of the method to be found
    :return: Whether the MethodInvocation matches the method name and has the `str_bool` value as its first argument.
    :rtype: bool
    """
    if not method_invocation.arguments or not isinstance(method_invocation.arguments[0], Literal):
        return False
    return (valid_method_invocation(method_invocation, method_name, num_arguments=1)
            and method_invocation.arguments[0].value == str_bool)


def webview_default_vulnerable(tree, method_name, issue_name, description, file_object, severity=Severity.WARNING):
    """
    A helper method for some WebView plugins that checks if a specific method is called with `method_name` and
    has a single argument that is the boolean value `false`. If so, it creates an issue with the other parameters.

    :param tree: javalang tree
    :param str method_name: method name to look for
    :param str issue_name: name of issue when it gets created
    :param str description: description for issue
    :param str file_object: current file being parsed
    :param Severity severity: severity of the issue
    :return: list of issues found in the `tree`
    :rtype: list
    """
    issues = []
    # iterate over method declarations to not go out of different scopes
    for _, method_declaration in tree.filter(MethodDeclaration):
        webviews = {}
        webview_name = None

        # webviews are vulnerable by default, get all created webviews
        for _, webview in method_declaration.filter(VariableDeclaration):
            # create a dictionary of webview and websettings objects keyed by their variable name to the
            #  AST parsed declaration
            webviews = add_webview_to_dict(webviews, webview, "WebView")

        # find if the webviews are vulnerable
        for _, method_invocation in method_declaration.filter(MethodInvocation):

            # check if method_name was called like
            # webview.getSettings().method_name("false"), which makes it not vulnerable
            if method_invocation.member == "getSettings":
                webview_name = method_invocation.qualifier

                # selectors are the .operators after the function, in this case `setAllowFileAccess`
                if method_invocation.selectors is None:
                    continue

                for selector in method_invocation.selectors:
                    if valid_set_method_bool(method_invocation=selector, str_bool="false", method_name=method_name):

                        # remove all instances of that webview from the dictionary as they are not vulnerable
                        if webviews.get(webview_name):
                            webviews = remove_dict_entry_by_value(webviews, webview_name)

            # check if method_name was called like
            # web_settings.method_name("false"), which makes it not vulnerable
            if (valid_set_method_bool(method_invocation=method_invocation, str_bool="false", method_name=method_name)
                  and webviews.get(webview_name)):
                webviews = remove_dict_entry_by_value(webviews, webview_name)


        # any webview left in the dictionary is vulnerable,
        # and since there can be many variables for a single webview,
        # we will convert the values from the webviews dictionary into a set to create issues
        for webview in set(webviews.values()):
            issues.append(Issue(category=CATEGORY, name=issue_name, severity=severity,
                                description=description, line_number=webview.position,
                                file_object=file_object))

    return issues


def add_webview_to_dict(webviews, webview, java_type):
    """
    Takes a `webview` and adds it to the `webviews` dictionary if the type of webview is `java_type`

    :param dict webviews: dictionary of already existing webviews that are found
    :param webview: AST parsed object
    :param java_type: the type of class the webview should be
    :return: dictionary with the webview added
    :rtype: dict
    """
    webviews_return = webviews
    if isinstance(webview.type, Type) and webview.type.name == java_type:
        webviews_return = deepcopy(webviews_return)
        for declaration in webview.declarators:
            webviews_return[declaration.name] = webview

    return webviews_return



import logging

from qark.issue import Severity, Issue
from qark.scanner.plugin import ManifestPlugin

log = logging.getLogger(__name__)

TASK_LAUNCH_MODE_DESCRIPTION = (
    "This results in AMS either resuming the earlier activity or loads it in a task with same affinity "
    "or the activity is started as a new task. This may result in Task Poisoning. "
    "https://www.usenix.org/system/files/conference/usenixsecurity15/sec15-paper-ren-chuangang.pdf"
)


class SingleTaskLaunchMode(ManifestPlugin):
    def __init__(self):
        super(ManifestPlugin, self).__init__(category="manifest", name="launchMode=singleTask found", description=TASK_LAUNCH_MODE_DESCRIPTION)
        self.severity = Severity.WARNING

    def run(self):
        with open(self.manifest_path, "r") as manifest_file:
            for line_number, line in enumerate(manifest_file):
                if 'android:launchMode="singleTask"' in line:
                    self.issues.append(Issue(
                        category=self.category, severity=self.severity, name=self.name,
                        description=self.description,
                        file_object=self.manifest_path,
                        line_number=line_number)
                    )


plugin = SingleTaskLaunchMode()



"""
Checks if either a method within ``ANDROID_LOGGING_METHODS`` is invoked with `Log.` before it.

For instance:
Log.d("test")
Log.e("test")

Both trigger but the following does not:
d("test")
e("test")
"""

import logging

import javalang
from javalang.tree import MethodInvocation

from qark.issue import Severity, Issue
from qark.plugins.helpers import java_files_from_files
from qark.scanner.plugin import JavaASTPlugin

log = logging.getLogger(__name__)

ANDROID_LOGGING_DESCRIPTION = (
    "Logs are detected. This may allow potential leakage of information from Android applications. Logs should never be"
    " compiled into an application except during development. Reference: "
    "https://developer.android.com/reference/android/util/Log.html"
)

ANDROID_LOGGING_METHODS = ("v", "d", "i", "w", "e")


class AndroidLogging(JavaASTPlugin):
    def __init__(self):
        super(AndroidLogging, self).__init__(category="file", name="Logging found",
                                             description=ANDROID_LOGGING_DESCRIPTION)
        self.severity = Severity.WARNING

    def run(self):
        for _, method_invocation in self.java_ast.filter(MethodInvocation):
            if method_invocation.qualifier == "Log" and method_invocation.member in ANDROID_LOGGING_METHODS:
                self.issues.append(Issue(
                    category=self.category, severity=self.severity, name=self.name,
                    description=self.description,
                    file_object=self.file_path,
                    line_number=method_invocation.position)
                )


plugin = AndroidLogging()



import logging
import re

from qark.issue import Severity, Issue
from qark.scanner.plugin import ManifestPlugin

log = logging.getLogger(__name__)

API_KEY_REGEX = re.compile(r'(?=.{20,})(?=.+\d)(?=.+[a-z])(?=.+[A-Z])')
SPECIAL_CHARACTER_REGEX = re.compile(r'(?=.+[!$%^~])')
HARDCODED_API_KEY_REGEX = re.compile(r'api_key|api|key', re.IGNORECASE)

API_KEY_DESCRIPTION = "Please confirm and investigate for potential API keys to determine severity."


class APIKeys(ManifestPlugin):
    def __init__(self):
        super(APIKeys, self).__init__(category="manifest", name="Potential API Key found",
                                      description=API_KEY_DESCRIPTION)
        self.severity = Severity.INFO

    def run(self):
        with open(self.manifest_path, "r") as manifest_file:
            for line_number, line in enumerate(manifest_file):
                # TODO: Fix API_KEY_REGEX, there are too many false positives
                # if re.search(API_KEY_REGEX, line) and not re.search(SPECIAL_CHARACTER_REGEX, line):
                #     self.issues.append(Issue(
                #         category=self.category, severity=self.severity, name=self.name,
                #         description=self.description,
                #         file_object=self.manifest_path,
                #         line_number=line_number)
                #     )
                if re.search(HARDCODED_API_KEY_REGEX, line):
                    self.issues.append(Issue(
                        category=self.category, severity=self.severity, name=self.name,
                        description=self.description,
                        file_object=self.manifest_path,
                        line_number=line_number)
                    )


plugin = APIKeys()



from setuptools import setup, find_packages
import os
import io

QARK_DIR = "qark"
LIB_DIR = os.path.join(QARK_DIR, "lib")


exploit_apk_files = [os.path.join(dir_path, filename).replace(os.path.join(QARK_DIR, ""), "")
                     for dir_path, _, files in os.walk(os.path.join(QARK_DIR, "exploit_apk"))
                     for filename in files]


with io.open('README.rst', 'rt', encoding='utf8') as f:
    long_description = f.read()

setup(
    name="qark",
    version="3.0.1",
    packages=find_packages(exclude=["tests*"]),
    package_dir={QARK_DIR: QARK_DIR},
    package_data={
        QARK_DIR: [
            os.path.join("lib", "decompilers", "*.jar"),  # include any decompiler jar files
            os.path.join("lib", "apktool", "*.jar"),  # include apktool
            os.path.join("lib", "dex2jar-2.0", "*"),  # include dex2jar
            os.path.join("lib", "dex2jar-2.0", "lib", "*"),  # include dex2jar
            os.path.join("templates", "*.jinja"),  # include the reporting template files
        ] + exploit_apk_files,  # include all the java files required for creating an exploit APK
    },
    install_requires=[
        "requests[security]",
        "pluginbase",
        "jinja2",
        "enum34; python_version < '3.4'",
        "javalang",
        "click",
        "six",
    ],
    # metadata for upload to PyPI
    author="Tushar Dalvi & Tony Trummer",
    author_email="tushardalvi@gmail.com, tonytrummer@hotmail.com",
    description="Android static code analyzer",
    long_description=long_description,
    license="Apache 2.0",
    keywords="android security qark exploit",
    url="https://www.github.com/linkedin/qark",
    entry_points="""
        [console_scripts]
        qark=qark.qark:cli""",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: Unix",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
    ]
)



import logging

from javalang.tree import MethodInvocation

from qark.issue import Issue, Severity
from qark.plugins.webview.helpers import webview_default_vulnerable, valid_set_method_bool
from qark.scanner.plugin import JavaASTPlugin, ManifestPlugin

log = logging.getLogger(__name__)

SET_ALLOW_UNIVERSAL_ACCESS_FROM_FILE_URLS_DESCRIPTION = (
    "JavaScript running in a file scheme context can access content from any origin. This is an insecure default "
    "value for minSdkVersion < 16 or may have been overridden (setAllowUniversalAccessFromFileURLs) in later versions. "
    "To validate this vulnerability, load the following local file in this WebView: "
    "file://qark/poc/html/UNIV_FILE_WARNING.html"
)


class SetAllowUniversalAccessFromFileURLs(JavaASTPlugin, ManifestPlugin):
    """This plugin checks if the `setAllowUniversalAccessFromFileURLs` method is called with a value of `true`, or
    if the default is vulnerable."""
    def __init__(self):
        super(SetAllowUniversalAccessFromFileURLs, self).__init__(category="webview",
                                                                  name="Webview enables universal access for JavaScript",
                                                                  description=SET_ALLOW_UNIVERSAL_ACCESS_FROM_FILE_URLS_DESCRIPTION)
        self.severity = Severity.WARNING
        self.java_method_name = "setAllowUniversalAccessFromFileURLs"

    def run(self):
        if self.min_sdk <= 15:
            self.issues.extend(webview_default_vulnerable(self.java_ast, method_name=self.java_method_name,
                                                          issue_name=self.name, description=self.description,
                                                          file_object=self.file_path, severity=self.severity))
        else:
            for _, method_invocation in self.java_ast.filter(MethodInvocation):
                if valid_set_method_bool(method_invocation, str_bool="true", method_name=self.java_method_name):
                    self.issues.append(Issue(category=self.category, name=self.name, severity=self.severity,
                                             description=self.description, line_number=method_invocation.position,
                                             file_object=self.file_path))


plugin = SetAllowUniversalAccessFromFileURLs()



import os
from functools import partial


def create_directories_to_path(path):
    """Create directories to a path if they don't exist."""

    try:
        os.makedirs(os.path.dirname(path))
    except Exception:
        # directory already exists
        pass


def file_has_extension(extension, file_path):
    return os.path.splitext(file_path.lower())[1] == extension.lower()


is_java_file = partial(file_has_extension, ".java")


def environ_path_variable_exists(variable_name):
    """Determines if the os.environ variable exists and is a valid path.

    :rtype: bool
    """
    try:
        return os.path.exists(os.environ[variable_name])
    except KeyError:
        return False



import logging

from qark.scanner.plugin import FileContentsPlugin
from qark.issue import Severity, Issue
from qark.plugins.helpers import run_regex

log = logging.getLogger(__name__)


class PackagedPrivateKeys(FileContentsPlugin):
    PRIVATE_KEY_REGEXES = (r'PRIVATE\sKEY',)

    def __init__(self):
        super(PackagedPrivateKeys, self).__init__(category="crypto",
                                                  name="Encryption keys are packaged with the application")

        self.severity = Severity.VULNERABILITY

    def run(self):
        for regex in PackagedPrivateKeys.PRIVATE_KEY_REGEXES:
            if run_regex(self.file_path, regex):
                log.debug("It appears there is a private key embedded in your application: %s", self.file_path)
                description = "It appears there is a private key embedded in your application in the following file:"
                self.issues.append(
                    Issue(self.category, self.name, self.severity, description, file_object=self.file_path))


plugin = PackagedPrivateKeys()



from qark.plugins.helpers import run_regex
from qark.scanner.plugin import FileContentsPlugin
from qark.issue import Severity, Issue

import logging

log = logging.getLogger(__name__)


WORLD_READABLE = "MODE_WORLD_READABLE"
WORLD_WRITEABLE = "MODE_WORLD_WRITEABLE"

WORLD_READABLE_DESCRIPTION = "World readable file found. Any application or file browser can access and read this file"
WORLD_WRITEABLE_DESCRIPTION = "World writeable file found. Any application or file browser can write to this file"


class FilePermissions(FileContentsPlugin):
    """
    This module runs a regex search on every Java file looking for `WORLD_READABLE` and `WORLD_WRITEABLE` modes.
    """
    def __init__(self):
        super(FilePermissions, self).__init__(category="file", name="File Permissions")
        self.severity = Severity.WARNING

    def run(self):
        if run_regex(self.file_path, WORLD_READABLE):
            self.issues.append(Issue(category=self.category, name="World readable file", severity=self.severity,
                                     description=WORLD_READABLE_DESCRIPTION, file_object=self.file_path))
        if run_regex(self.file_path, WORLD_WRITEABLE):
            self.issues.append(Issue(category=self.category, name="World writeable file", severity=self.severity,
                                     description=WORLD_WRITEABLE_DESCRIPTION, file_object=self.file_path))


plugin = FilePermissions()



import logging

import javalang
from javalang.tree import MethodDeclaration, MethodInvocation, ReturnStatement, StatementExpression

from qark.issue import Issue, Severity
from qark.scanner.plugin import JavaASTPlugin

log = logging.getLogger(__name__)

CERT_METHODS = ("checkServerTrusted", "onReceivedSslError")
CHECK_SERVER_TRUSTED = "Instance of checkServerTrusted, with empty body found. "
ON_RECEIVED_SSL_DESC = ("Specifically, the implementation ignores all SSL certificate validation errors, making your "
                        "app vulnerable to man-in-the-middle attacks. To properly handle SSL certificate validation, "
                        "change your code to invoke SslErrorHandler.cancel(). For details, please see: "
                        "https://developer.android.com/reference/android/webkit/WebViewClient.html")
MITM_DESCRIPTION = ("This means this application is likely "
                    "vulnerable to Man-In-The-Middle attacks. This can be confirmed using the free version of "
                    "Burpsuite. Simply set the Android device's proxy to use Burpsuite via the network settings, "
                    "but DO NOT install the Portswigger CA certificate on the device. If you still see traffic in the "
                    "proxy, the app is vulnerable. Note: You need to ensure you exercise this code path. If you are "
                    "unsure, make sure you click through each part of the application which makes network requests. "
                    "You may need to toggle the proxy on/off to get past sections that do validate certificates "
                    "properly in order to reach the vulnerable code. This proves that it will accept certificates "
                    "from any CA. You should always validate your configuration by visiting an HTTPS site in the "
                    "native browser and verifying you receive a certificate warning. For details, please see: "
                    "https://developer.android.com/training/articles/security-ssl.html")


class CertValidation(JavaASTPlugin):
    """
    This plugin checks if a method in `CERT_METHODS` is overriden with an insecure version that usually does not verify
    SSL connections.
    """
    def __init__(self):
        super(CertValidation, self).__init__(category="cert", name="Certification Validation")
        self.severity = Severity.WARNING

    def run(self):
        cert_methods = (method_declaration for _, method_declaration in self.java_ast.filter(MethodDeclaration)
                        if method_declaration.name in CERT_METHODS)

        for cert_method in cert_methods:
            if cert_method.name == "checkServerTrusted":
                self._check_server_trusted(cert_method, self.file_path)
            elif cert_method.name == "onReceivedSslError":
                self._on_received_ssl_error(cert_method, self.file_path)

    def _check_server_trusted(self, cert_method, current_file):
        """
        Determines if the `checkServerTrusted` method is overriden with either no function body (defaults to allow)
        or if the body just returns.
        """
        if not cert_method.body:
            self.issues.append(Issue(category=self.category, name="Empty certificate method",
                                     severity=self.severity,
                                     description=CHECK_SERVER_TRUSTED + MITM_DESCRIPTION,
                                     file_object=current_file,
                                     line_number=cert_method.position))
        elif len(cert_method.body) == 1 and type(cert_method.body[0]) is ReturnStatement:
            self.issues.append(Issue(category=self.category, name="Empty (return) certificate method",
                                     severity=self.severity,
                                     description=CHECK_SERVER_TRUSTED + MITM_DESCRIPTION,
                                     file_object=current_file,
                                     line_number=cert_method.position))

    def _on_received_ssl_error(self, cert_method, current_file):
        """
        Determines if the `onReceivedSslError` method is overriden with a body of `handler.proceed()` which will
        drop SSL errors.
        """
        if cert_method.body and len(cert_method.body) == 1 and type(cert_method.body[0]) is StatementExpression:
            for _, method_invocation in cert_method.filter(MethodInvocation):
                if method_invocation.member == "proceed":
                    self.issues.append(Issue(category=self.category, name="Unsafe implementation of onReceivedSslError",
                                             severity=self.severity,
                                             description=ON_RECEIVED_SSL_DESC,
                                             file_object=current_file,
                                             line_number=method_invocation.position))


plugin = CertValidation()



"""This plugin checks if there are any lines in the file that match the regex ``API_KEY_REGEX`` while
also not matching ``SPECIAL_CHARACTER_REGEX``."""

import logging
import re

from qark.issue import Severity, Issue
from qark.scanner.plugin import FileContentsPlugin

log = logging.getLogger(__name__)

API_KEY_REGEX = re.compile(r'(?=.{20,})(?=.+\d)(?=.+[a-z])(?=.+[A-Z])(?=.+[-_])')
SPECIAL_CHARACTER_REGEX = re.compile(r'(?=.+[!$%^&*()_+|~=`{}\[\]:<>?,./])')
BLACKLISTED_EXTENSIONS = (".apk", ".dex", ".png", ".jar")

API_KEY_DESCRIPTION = "Please confirm and investigate the API key to determine its severity."


class JavaAPIKeys(FileContentsPlugin):
    def __init__(self):
        super(JavaAPIKeys, self).__init__(category="file", name="Potential API Key found",
                                          description=API_KEY_DESCRIPTION)
        self.severity = Severity.INFO

    def run(self):
        if any(self.file_path.endswith(extension) for extension in BLACKLISTED_EXTENSIONS):
            return

        for line_number, line in enumerate(self.file_contents.split("\n")):
            for word in line.split():
                if re.search(API_KEY_REGEX, word) and not re.search(SPECIAL_CHARACTER_REGEX, word):
                    self.issues.append(Issue(
                        category=self.category, severity=self.severity, name=self.name,
                        description=self.description,
                        file_object=self.file_path,
                        line_number=(line_number, 0))
                    )


plugin = JavaAPIKeys()



import logging

from qark.issue import Severity, Issue
from qark.scanner.plugin import ManifestPlugin

log = logging.getLogger(__name__)

PATH_USAGE_DESCRIPTION = ("android:path means that the permission applies to the exact path declared"
                          " in android:path. This expression does not protect the sub-directories")


class AndroidPath(ManifestPlugin):
    def __init__(self):
        super(AndroidPath, self).__init__(category="manifest", name="android:path tag used",
                                          description=PATH_USAGE_DESCRIPTION)

        self.severity = Severity.WARNING

    def run(self):
        with open(self.manifest_path, "r") as manifest_file:
            for line_number, line in enumerate(manifest_file):
                if "android:path=" in line:
                    self.issues.append(Issue(
                        category=self.category, severity=self.severity, name=self.name,
                        description=self.description,
                        file_object=self.manifest_path,
                        line_number=line_number)
                    )


plugin = AndroidPath()



import abc
import os
import logging
from xml.dom import minidom

from pluginbase import PluginBase
import javalang
from qark.utils import is_java_file
from qark.plugins.manifest_helpers import get_min_sdk, get_target_sdk, get_package_from_manifest

log = logging.getLogger(__name__)


plugin_base = PluginBase(package="qark.custom_plugins")

# plugin modules to blacklist, `helpers` should always be blacklisted as it is not a plugin
BLACKLISTED_PLUGIN_MODULES = {"helpers"}


def get_plugin_source(category=None):
    """
    Returns a `PluginBase.PluginSource` based on the `category`.
    :param category: plugin category, subdirectory under `plugins/`
    :return: `PluginBase.PluginSource`
    """
    path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", "plugins")
    if category is not None:
        path = os.path.join(path, category)

    try:
        return plugin_base.make_plugin_source(searchpath=[path], persist=True)
    except Exception:
        log.exception("Failed to get all plugins. Is the file path to plugins %s correct?", path)
        raise SystemExit("Failed to get all plugins. Is the file path correct?")


def get_plugins(category=None):
    """
    Returns all plugins defined by a `category` and removes plugins defined in ``BLACKLISTED_PLUGIN_MODULES``.

    :param category: plugin category, subdirectory under `plugins/`
    :return: modules for that category
    :rtype: list
    """
    plugins = get_plugin_source(category=category).list_plugins()

    return [plugin for plugin in plugins if plugin not in BLACKLISTED_PLUGIN_MODULES]


class BasePlugin(object):
    __metaclass__ = abc.ABCMeta

    def __init__(self, name, category, description=None, **kwargs):
        self.category = category
        self.name = name
        self.description = description

        super(BasePlugin, self).__init__(**kwargs)

        self.issues = []

    @abc.abstractmethod
    def run(self):
        """
        Method to be called for each plugin to add issues.

        :param List[str] files: a list of files gathered by `Scanner` as absolute paths
        :param dict apk_constants: dictionary containing extra information
                                    that some plugins can use (min_sdk, target_sdk)
        """
        pass


class PluginObserver(BasePlugin):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def update(self, *args, **kwargs):
        """For plugins to implement."""

    @abc.abstractmethod
    def reset(self):
        """Clear any class attributes that were set for the file."""


class FilePathPlugin(PluginObserver):
    """Subclass this plugin if your plugin needs action based on a new file path."""
    __metaclass__ = abc.ABCMeta

    file_path = None
    has_been_set = False

    def update(self, file_path, call_run=False):
        """
        :param str file_path:
        :param bool call_run: Whether or not `self.run()` should be called. Prevents `run()` being called multiple times
        """
        if not file_path:
            FilePathPlugin.file_path = None
            return

        if not self.has_been_set:
            FilePathPlugin.file_path = file_path
            FilePathPlugin.has_been_set = True

        if call_run:
            self.run()

    @classmethod
    def reset(cls):
        FilePathPlugin.file_path = None
        FilePathPlugin.has_been_set = False


class FileContentsPlugin(FilePathPlugin):
    """Subclass this plugin if your plugin needs to operate on file contents and no other plugin type suffices."""
    __metaclass__ = abc.ABCMeta

    file_contents = None
    readable = True

    def update(self, file_path, call_run=False):
        """
        If `file_contents` is None, then the file has not been read yet, and we must set it.
        If the file is not able to be read then we set a flag telling other plugins to not try to operate on it.
        """
        if not self.readable:
            return

        if self.file_contents is None:
            # Make sure the file path has been set.
            super(FileContentsPlugin, self).update(file_path)

            try:
                with open(self.file_path, "r") as f:
                    FileContentsPlugin.file_contents = f.read()

            except IOError:
                log.debug("Unable to operate on file %s for reading", self.file_path)
                FileContentsPlugin.readable = False
                FileContentsPlugin.file_contents = None
                return

            except UnicodeDecodeError:
                # Try to read file in ISO-8859-1 encoding, used for png, xml, webp, some other resource files etc
                try:
                    with open(self.file_path, "r", encoding="ISO-8859-1") as f:
                        FileContentsPlugin.file_contents = f.read()

                except Exception:
                    # Give up on the file
                    log.debug("Unable to operate on file %s", self.file_path)
                    FileContentsPlugin.readable = False
                    FileContentsPlugin.file_contents = None
                    return

        if call_run and self.file_contents is not None:
            self.run()

    @classmethod
    def reset(cls):
        FileContentsPlugin.file_contents = None
        FileContentsPlugin.readable = True

        super(FileContentsPlugin, cls).reset()


class JavaASTPlugin(FileContentsPlugin):
    __metaclass__ = abc.ABCMeta

    java_ast = None
    parseable = True

    def update(self, file_path, call_run=False):
        if not self.parseable:
            return

        if self.java_ast is None and is_java_file(file_path):
            # Make sure the file contents have been set
            super(JavaASTPlugin, self).update(file_path, call_run=False)

            if self.file_contents:
                try:
                    JavaASTPlugin.java_ast = javalang.parse.parse(self.file_contents)
                except (javalang.parser.JavaSyntaxError, IndexError):
                    log.debug("Unable to parse AST for file %s", self.file_path)
                    JavaASTPlugin.java_ast = None
                    JavaASTPlugin.parseable = False
                    return

        if call_run and self.java_ast is not None:
            self.run()

    @classmethod
    def reset(cls):
        JavaASTPlugin.java_ast = None
        JavaASTPlugin.parseable = True

        super(JavaASTPlugin, cls).reset()


class ManifestPlugin(BasePlugin):
    __metaclass__ = abc.ABCMeta

    manifest_xml = None
    manifest_path = None
    min_sdk = -1
    target_sdk = -1
    package_name = "PACKAGE_NOT_FOUND"

    @classmethod
    def update_manifest(cls, path_to_manifest):
        """Users of this class should call this method instead of changing class attributes directly"""
        cls.manifest_path = path_to_manifest
        try:
            cls.manifest_xml = minidom.parse(path_to_manifest)
        except Exception:
            # path_to_manifest is None or has bad XML
            cls.manifest_xml = None
            log.debug("Failed to update manifest for file %s", path_to_manifest)
            return

        try:
            cls.min_sdk = get_min_sdk(cls.manifest_path)
            cls.target_sdk = get_target_sdk(cls.manifest_path)
        except AttributeError:
            # manifest path is not set, assume min_sdk and target_sdk
            cls.min_sdk = cls.target_sdk = 1

        try:
            cls.package_name = get_package_from_manifest(cls.manifest_path)
        except IOError:
            cls.package_name = "PACKAGE_NOT_FOUND"

    @abc.abstractmethod
    def run(self):
        """User should define how their plugin runs"""



from qark.plugins.manifest_helpers import get_min_sdk, get_target_sdk, get_package_from_manifest
from qark.plugins.helpers import java_files_from_files
from qark.scanner.plugin import ManifestPlugin
from qark.issue import Severity, Issue

import os
import logging

from enum import Enum
import javalang
from javalang.tree import Literal, MethodDeclaration, MethodInvocation

log = logging.getLogger(__name__)

COMPONENT_ENTRIES = {"activity": {"onCreate", "onStart"},
                     "activity-alias": {"onCreate", "onStart"},
                     "receiver": {"onReceive", },
                     "service": {"onCreate", "onBind", "onStartCommand", "onHandleIntent"},
                     "provider": {"onReceive", }}

# it seems like get, getInt, getString are the ones that are used for decompiled APKs
EXTRAS_METHOD_NAMES = [
    'getExtras',
    'getStringExtra',
    'getIntExtra',
    'getIntArrayExtra',
    'getFloatExtra',
    'getFloatArrayExtra',
    'getDoubleExtra',
    'getDoubleArrayExtra',
    'getCharExtra',
    'getCharArrayExtra',
    'getByteExtra',
    'getByteArrayExtra',
    'getBundleExtra',
    'getBooleanExtra',
    'getBooleanArrayExtra',
    'getCharSequenceArrayExtra',
    'getCharSequenceArrayListExtra',
    'getCharSequenceExtra',
    'getIntegerArrayListExtra',
    'getLongArrayExtra',
    'getLongExtra',
    'getParcelableArrayExtra',
    'getParcelableArrayListExtra',
    'getParcelableExtra',
    'getSerializableExtra',
    'getShortArrayExtra',
    'getShortExtra',
    'getStringArrayExtra',
    'getStringArrayListExtra',
    'getString',
    'getInt',
]

PROTECTED_BROADCASTS = ['android.intent.action.SCREEN_OFF', 'android.intent.action.SCREEN_ON',
                        'android.intent.action.USER_PRESENT', 'android.intent.action.TIME_TICK',
                        'android.intent.action.TIMEZONE_CHANGED', 'android.intent.action.BOOT_COMPLETED',
                        'android.intent.action.PACKAGE_INSTALL', 'android.intent.action.PACKAGE_ADDED',
                        'android.intent.action.PACKAGE_REPLACED', 'android.intent.action.MY_PACKAGE_REPLACED',
                        'android.intent.action.PACKAGE_REMOVED', 'android.intent.action.PACKAGE_FULLY_REMOVED',
                        'android.intent.action.PACKAGE_CHANGED', 'android.intent.action.PACKAGE_RESTARTED',
                        'android.intent.action.PACKAGE_DATA_CLEARED', 'android.intent.action.PACKAGE_FIRST_LAUNCH',
                        'android.intent.action.PACKAGE_NEEDS_VERIFICATION', 'android.intent.action.PACKAGE_VERIFIED',
                        'android.intent.action.UID_REMOVED', 'android.intent.action.QUERY_PACKAGE_RESTART',
                        'android.intent.action.CONFIGURATION_CHANGED', 'android.intent.action.LOCALE_CHANGED',
                        'android.intent.action.BATTERY_CHANGED', 'android.intent.action.BATTERY_LOW',
                        'android.intent.action.BATTERY_OKAY', 'android.intent.action.ACTION_POWER_CONNECTED',
                        'android.intent.action.ACTION_POWER_DISCONNECTED', 'android.intent.action.ACTION_SHUTDOWN',
                        'android.intent.action.DEVICE_STORAGE_LOW', 'android.intent.action.DEVICE_STORAGE_OK',
                        'android.intent.action.DEVICE_STORAGE_FULL', 'android.intent.action.DEVICE_STORAGE_NOT_FULL',
                        'android.intent.action.NEW_OUTGOING_CALL', 'android.intent.action.REBOOT',
                        'android.intent.action.DOCK_EVENT', 'android.intent.action.MASTER_CLEAR_NOTIFICATION',
                        'android.intent.action.USER_ADDED', 'android.intent.action.USER_REMOVED',
                        'android.intent.action.USER_STOPPED', 'android.intent.action.USER_BACKGROUND',
                        'android.intent.action.USER_FOREGROUND', 'android.intent.action.USER_SWITCHED',
                        'android.app.action.ENTER_CAR_MODE', 'android.app.action.EXIT_CAR_MODE',
                        'android.app.action.ENTER_DESK_MODE', 'android.app.action.EXIT_DESK_MODE',
                        'android.appwidget.action.APPWIDGET_UPDATE_OPTIONS',
                        'android.appwidget.action.APPWIDGET_DELETED', 'android.appwidget.action.APPWIDGET_DISABLED',
                        'android.appwidget.action.APPWIDGET_ENABLED', 'android.backup.intent.RUN',
                        'android.backup.intent.CLEAR', 'android.backup.intent.INIT',
                        'android.bluetooth.adapter.action.STATE_CHANGED',
                        'android.bluetooth.adapter.action.SCAN_MODE_CHANGED',
                        'android.bluetooth.adapter.action.DISCOVERY_STARTED',
                        'android.bluetooth.adapter.action.DISCOVERY_FINISHED',
                        'android.bluetooth.adapter.action.LOCAL_NAME_CHANGED',
                        'android.bluetooth.adapter.action.CONNECTION_STATE_CHANGED',
                        'android.bluetooth.device.action.FOUND', 'android.bluetooth.device.action.DISAPPEARED',
                        'android.bluetooth.device.action.CLASS_CHANGED',
                        'android.bluetooth.device.action.ACL_CONNECTED',
                        'android.bluetooth.device.action.ACL_DISCONNECT_REQUESTED',
                        'android.bluetooth.device.action.ACL_DISCONNECTED',
                        'android.bluetooth.device.action.NAME_CHANGED',
                        'android.bluetooth.device.action.BOND_STATE_CHANGED',
                        'android.bluetooth.device.action.NAME_FAILED',
                        'android.bluetooth.device.action.PAIRING_REQUEST',
                        'android.bluetooth.device.action.PAIRING_CANCEL',
                        'android.bluetooth.device.action.CONNECTION_ACCESS_REPLY',
                        'android.bluetooth.headset.profile.action.CONNECTION_STATE_CHANGED',
                        'android.bluetooth.headset.profile.action.AUDIO_STATE_CHANGED',
                        'android.bluetooth.headset.action.VENDOR_SPECIFIC_HEADSET_EVENT',
                        'android.bluetooth.a2dp.profile.action.CONNECTION_STATE_CHANGED',
                        'android.bluetooth.a2dp.profile.action.PLAYING_STATE_CHANGED',
                        'android.bluetooth.input.profile.action.CONNECTION_STATE_CHANGED',
                        'android.bluetooth.pan.profile.action.CONNECTION_STATE_CHANGED',
                        'android.hardware.display.action.WIFI_DISPLAY_STATUS_CHANGED',
                        'android.hardware.usb.action.USB_STATE', 'android.hardware.usb.action.USB_ACCESSORY_ATTACHED',
                        'android.hardware.usb.action.USB_ACCESSORY_ATTACHED',
                        'android.hardware.usb.action.USB_DEVICE_ATTACHED',
                        'android.hardware.usb.action.USB_DEVICE_DETACHED', 'android.intent.action.HEADSET_PLUG',
                        'android.intent.action.ANALOG_AUDIO_DOCK_PLUG', 'android.intent.action.DIGITAL_AUDIO_DOCK_PLUG',
                        'android.intent.action.HDMI_AUDIO_PLUG', 'android.intent.action.USB_AUDIO_ACCESSORY_PLUG',
                        'android.intent.action.USB_AUDIO_DEVICE_PLUG', 'android.net.conn.CONNECTIVITY_CHANGE',
                        'android.net.conn.CONNECTIVITY_CHANGE_IMMEDIATE', 'android.net.conn.DATA_ACTIVITY_CHANGE',
                        'android.net.conn.BACKGROUND_DATA_SETTING_CHANGED',
                        'android.net.conn.CAPTIVE_PORTAL_TEST_COMPLETED', 'android.nfc.action.LLCP_LINK_STATE_CHANGED',
                        'com.android.nfc_extras.action.RF_FIELD_ON_DETECTED',
                        'com.android.nfc_extras.action.RF_FIELD_OFF_DETECTED',
                        'com.android.nfc_extras.action.AID_SELECTED', 'android.nfc.action.TRANSACTION_DETECTED',
                        'android.intent.action.CLEAR_DNS_CACHE', 'android.intent.action.PROXY_CHANGE',
                        'android.os.UpdateLock.UPDATE_LOCK_CHANGED', 'android.intent.action.DREAMING_STARTED',
                        'android.intent.action.DREAMING_STOPPED', 'android.intent.action.ANY_DATA_STATE',
                        'com.android.server.WifiManager.action.START_SCAN',
                        'com.android.server.WifiManager.action.DELAYED_DRIVER_STOP',
                        'android.net.wifi.WIFI_STATE_CHANGED', 'android.net.wifi.WIFI_AP_STATE_CHANGED',
                        'android.net.wifi.WIFI_SCAN_AVAILABLE', 'android.net.wifi.SCAN_RESULTS',
                        'android.net.wifi.RSSI_CHANGED', 'android.net.wifi.STATE_CHANGE',
                        'android.net.wifi.LINK_CONFIGURATION_CHANGED', 'android.net.wifi.CONFIGURED_NETWORKS_CHANGE',
                        'android.net.wifi.supplicant.CONNECTION_CHANGE', 'android.net.wifi.supplicant.STATE_CHANGE',
                        'android.net.wifi.p2p.STATE_CHANGED', 'android.net.wifi.p2p.DISCOVERY_STATE_CHANGE',
                        'android.net.wifi.p2p.THIS_DEVICE_CHANGED', 'android.net.wifi.p2p.PEERS_CHANGED',
                        'android.net.wifi.p2p.CONNECTION_STATE_CHANGE',
                        'android.net.wifi.p2p.PERSISTENT_GROUPS_CHANGED', 'android.net.conn.TETHER_STATE_CHANGED',
                        'android.net.conn.INET_CONDITION_ACTION',
                        'android.intent.action.EXTERNAL_APPLICATIONS_AVAILABLE',
                        'android.intent.action.EXTERNAL_APPLICATIONS_UNAVAILABLE',
                        'android.intent.action.AIRPLANE_MODE', 'android.intent.action.ADVANCED_SETTINGS',
                        'android.intent.action.BUGREPORT_FINISHED',
                        'android.intent.action.ACTION_IDLE_MAINTENANCE_START',
                        'android.intent.action.ACTION_IDLE_MAINTENANCE_END', 'android.intent.action.SERVICE_STATE',
                        'android.intent.action.RADIO_TECHNOLOGY',
                        'android.intent.action.EMERGENCY_CALLBACK_MODE_CHANGED', 'android.intent.action.SIG_STR',
                        'android.intent.action.ANY_DATA_STATE', 'android.intent.action.DATA_CONNECTION_FAILED',
                        'android.intent.action.SIM_STATE_CHANGED', 'android.intent.action.NETWORK_SET_TIME',
                        'android.intent.action.NETWORK_SET_TIMEZONE',
                        'android.intent.action.ACTION_SHOW_NOTICE_ECM_BLOCK_OTHERS',
                        'android.intent.action.ACTION_MDN_STATE_CHANGED',
                        'android.provider.Telephony.SPN_STRINGS_UPDATED', 'android.provider.Telephony.SIM_FULL',
                        'com.android.internal.telephony.data-restart-trysetup',
                        'com.android.internal.telephony.data-stall']

EXPORTED_AND_PERMISSION_TAG = ("The {tag} {tag_name} tag is exported and protected by a permission, "
                               "but the permission can be obtained by malicious apps installed "
                               "prior to this one. More info: "
                               "https://github.com/commonsguy/cwac-security/blob/master/PERMS.md. "
                               "Failing to protect {tag} tags could leave them vulnerable to attack "
                               "by malicious apps. The {tag} tags should be reviewed for "
                               "vulnerabilities, such as injection and information leakage.")
EXPORTED_IN_PROTECTED = ("The {tag} {tag_name} is exported, but the associated Intents can only be sent "
                         "by SYSTEM level apps. They could still potentially be vulnerable, "
                         "if the Intent carries data that is tainted (2nd order injection)")
EXPORTED = ("The {tag} {tag_name} is exported, but not protected by any permissions. Failing to protect "
            "{tag} tags could leave them vulnerable to attack by malicious apps. The "
            "{tag} tags should be reviewed for vulnerabilities, such as injection and information leakage.")

EXPORTED_TAGS_ISSUE_NAME = "Exported tags"


class Receiver(Enum):
    id = 1
    type = "receiver"
    parent = "exportedReceivers"


class Provider(Enum):
    id = 2
    type = "provider"
    parent = "exportedContentProviders"


class Activity(Enum):
    id = 3
    type = "activity"
    parent = "exportedActivities"


class Broadcast(Enum):
    id = 4
    type = "broadcast"
    parent = "exportedBroadcasts"


class Service(Enum):
    id = 5
    parent = "exportedServices"
    type = "service"


TAG_INFO = {"receiver": Receiver, "provider": Provider, "activity": Activity,
            "activity-alias": Activity, "service": Service}


class ExportedTags(ManifestPlugin):
    all_files = None  # Put here to not break Liskov Substitution with the other ManifestPlugins
    # This plugin is VERY abnormal from the other manifest plugins as it needs to search
    #   java files in order to create the exploit APK. Be very careful around here.

    def __init__(self):
        super(ExportedTags, self).__init__(category="manifest", name=EXPORTED_TAGS_ISSUE_NAME)

        self.bad_exported_tags = ("activity", "activity-alias", "service", "receiver", "provider")

    def run(self):
        for tag in self.bad_exported_tags:
            all_tags_of_type_tag = self.manifest_xml.getElementsByTagName(tag)
            for possibly_vulnerable_tag in all_tags_of_type_tag:
                self._check_manifest_issues(possibly_vulnerable_tag, tag, self.manifest_path)

        self._add_exported_tags_arguments_to_issue(list(java_files_from_files(self.all_files)))

    def _check_manifest_issues(self, possibly_vulnerable_tag, tag, file_object):
        """
        Check exported tags for vulnerabilities, warnings, or information.

        :param possibly_vulnerable_tag: the tag in as an XML object
        :param str tag: the tag name
        :param file_object: the object of the file being read (manifest)
        """
        is_exported = "android:exported" in possibly_vulnerable_tag.attributes.keys()
        has_permission = "android:permission" in possibly_vulnerable_tag.attributes.keys()
        has_intent_filters = len(possibly_vulnerable_tag.getElementsByTagName("intent-filter")) > 0
        exported = possibly_vulnerable_tag.attributes.get("android:exported").value.lower() if is_exported else None
        tag_is_provider = tag.lower() == "provider"
        try:
            tag_name = possibly_vulnerable_tag.attributes.get("android:name").value
        except AttributeError:
            tag_name = None

        info_enum = TAG_INFO[tag]

        if exported == "false":
            # activity is not exported
            return

        if (exported is not None and exported != "false") or tag_is_provider:
            if tag_is_provider and self.min_sdk > 16 or self.target_sdk > 16:
                # provider is not vulnerable under these conditions, return
                return

            if has_permission and self.min_sdk < 20:
                # exported tag with permission
                self.issues.append(Issue(category="Manifest", name=self.name,
                                         severity=Severity.INFO,
                                         description=EXPORTED_AND_PERMISSION_TAG.format(tag=tag, tag_name=tag_name),
                                         file_object=file_object, apk_exploit_dict={"exported_enum": info_enum,
                                                                                    "tag_name": tag_name,
                                                                                    "package_name": self.package_name}))
            elif exported and not has_intent_filters:
                # exported tag with no intent filters
                self.issues.append(Issue(category="Manifest", name=self.name,
                                         severity=Severity.WARNING,
                                         description=EXPORTED.format(tag=tag, tag_name=tag_name),
                                         file_object=file_object, apk_exploit_dict={"exported_enum": info_enum,
                                                                                    "tag_name": tag_name,
                                                                                    "package_name": self.package_name}))
        for intent_filter in possibly_vulnerable_tag.getElementsByTagName("intent-filter"):
            for action in intent_filter.getElementsByTagName("action"):
                try:
                    protected = action.attributes["android:name"].value in PROTECTED_BROADCASTS
                except KeyError:
                    log.debug("Action doesn't have a name field, continuing execution")
                    continue

                if protected:
                    # intent filter has protected actions
                    name = "Protected Exported Tags"
                    description = EXPORTED_IN_PROTECTED.format(tag=tag, tag_name=tag_name)
                    severity = Severity.INFO

                elif has_permission and self.min_sdk < 20:
                    name = "Exported Tag With Permission"
                    description = EXPORTED_AND_PERMISSION_TAG.format(tag=tag, tag_name=tag_name)
                    severity = Severity.INFO

                else:
                    name = self.name
                    description = EXPORTED.format(tag=tag, tag_name=tag_name)
                    severity = Severity.WARNING

                self.issues.append(Issue(category="Manifest", name=name, severity=severity, description=description,
                                         file_object=file_object, apk_exploit_dict={"exported_enum": info_enum,
                                                                                    "tag_name": tag_name,
                                                                                    "package_name": self.package_name}))

    def _add_exported_tags_arguments_to_issue(self, java_files):
        """
        This helper method iterates over all the issues and searches for a corresponding java file for the tag.
        Once the java file has been found, it finds the method declaration based on tag type and then finds a method
        invocation in `EXTRAS_METHOD_NAMES` to pull arguments from.

        Once found it updates the `issue` with its arguments.

        :param List[str] java_files: list of java file paths
        """
        for issue in self.issues:
            try:
                file_name = issue.apk_exploit_dict["tag_name"].replace(".", os.sep)
            except IndexError:
                log.exception("Tag name %s does not have a period in it", issue.apk_exploit_dict["tag_name"])
                return

            self._get_arguments_for_method_from_file(java_files, issue, name_to_search_for=file_name)

    def _get_arguments_for_method_from_file(self, java_files, issue, name_to_search_for):
        """Confusing code ported over from QARK v1. Used to get arguments for QARK's exploit APK."""
        added = False
        for java_file in java_files:
            if name_to_search_for not in java_file:
                continue

            try:
                with open(java_file, "r") as java_file_to_read:
                    file_contents = java_file_to_read.read()
            except IOError:
                log.debug("Error reading file %s", java_file)
                continue

            try:
                parsed_tree = javalang.parse.parse(file_contents)
            except (javalang.parser.JavaSyntaxError, IndexError):
                log.debug("Error parsing file %s, continuing", java_file)
                continue

            issue.apk_exploit_dict["arguments"] = []
            # find method declarations for the component type
            for _, method_declaration in parsed_tree.filter(MethodDeclaration):
                if method_declaration.name in COMPONENT_ENTRIES[issue.apk_exploit_dict["exported_enum"].type.value]:

                    # find method invocations calling items that might have arguments we want to get
                    for _, method_invocation in parsed_tree.filter(MethodInvocation):
                        if method_invocation.member in EXTRAS_METHOD_NAMES:
                            if not method_invocation.arguments:
                                continue

                            for argument in method_invocation.arguments:
                                if not isinstance(argument, Literal):
                                    continue

                                if argument.value not in issue.apk_exploit_dict["arguments"]:
                                    added = True
                                    issue.apk_exploit_dict["arguments"].append(argument.value)
            if added:
                return


plugin = ExportedTags()



from qark.scanner.plugin import ManifestPlugin
from qark.issue import Severity, Issue

import logging

log = logging.getLogger(__name__)


class ManifestBackupAllowed(ManifestPlugin):
    def __init__(self):
        super(ManifestBackupAllowed, self).__init__(category="manifest", name="Backup is allowed in manifest",
                                                    description=(
                                                        "Backups enabled: Potential for data theft via local attacks via adb "
                                                        "backup, if the device has USB debugging enabled (not common). "
                                                        "More info: "
                                                        "http://developer.android.com/reference/android/R.attr.html#allowBackup"))

        self.severity = Severity.WARNING

    def run(self):
        application_sections = self.manifest_xml.getElementsByTagName("application")

        for application in application_sections:
            if "android:allowBackup" in application.attributes.keys():
                self.issues.append(Issue(category=self.category, severity=self.severity,
                                         name=self.name, description=self.description,
                                         file_object=self.manifest_path))


plugin = ManifestBackupAllowed()



from xml.etree import ElementTree

import logging

log = logging.getLogger(__name__)


def write_key_value_to_xml(key, value, path):
    """
    Checks if `key` exists in the parsed XML `path`, if it does not it creates a new
    element and appends it to the XML tree and then updates the file.

    :param key:
    :param value:
    :param path: Union[str, pathlib.Path, FileObject]
    :return:
    """
    try:
        xml_to_write = ElementTree.parse(path)
    except IOError:
        log.exception("Strings file for exploit APK does not exist")
        raise SystemExit("Strings file for exploit APK does not exist")

    if not xml_to_write.find(key):
        new_element = ElementTree.SubElement(xml_to_write.getroot(), "string", attrib={"name": key})
        new_element.text = value

        xml_to_write.write(path)


def write_key_value_to_string_array_xml(array_name, value, path, add_id=True):
    """
    Checks if `array_name` is name of a `string-array`, if it does not exist it creates a new
    element and appends it to the XML tree and then updates the file, if it exists it updates the existing element.

    :param str array_name:
    :param str value:
    :param Union[str, pathlib.Path, FileObject] path:
    :param bool add_id: Whether or not to create an id like `value{id}` where `id` is an int
    """
    try:
        strings_xml = ElementTree.parse(path)
    except IOError:
        log.exception("Extra keys file for exploit APK does not exist")
        raise SystemExit("Extra keys file for exploit APK does not exist")

    if add_id:
        last_id = 0

    # attempt to update the entry if it exists
    for string_array in strings_xml.findall("string-array"):
        if string_array.attrib.get("name") == array_name:
            if add_id:
                for child in string_array.getchildren():
                    last_id = child.text.split(value)[1]
                value = "{value}{id}".format(value=value, id=int(last_id)+1)

            sub_element_item = ElementTree.SubElement(string_array, "item")
            sub_element_item.text = value

            strings_xml.write(path)
            return value

    if add_id:
        value = "{value}{id}".format(value=value, id=last_id+1)

    # write the entry as it does not exist
    new_string_array = ElementTree.SubElement(strings_xml.getroot(), "string-array", attrib={"name": array_name})
    sub_element_item = ElementTree.SubElement(new_string_array, "item")
    sub_element_item.text = value

    strings_xml.write(path)
    return value


def get_manifest_out_of_files(files):
    """
    Parses `files` for a file that ends with `androidmanifest.xml`.
    :param Set[str] files: list of paths to files as absolute paths
    :return: manifest string if in `files`, else None
    """
    for file_name in files:
        if file_name.lower().endswith("androidmanifest.xml"):
            return file_name
    return None
